/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: AbstractDeploymentCore.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.core;

import java.io.File;
import org.apache.maven.plugin.MojoExecutionException;
import org.ow2.util.maven.plugin.deployment.api.IDeploymentBehavior;
import org.ow2.util.maven.plugin.deployment.api.IDeploymentMojo;
import org.ow2.util.maven.plugin.deployment.api.IDeploymentCore;
import org.ow2.util.maven.plugin.deployment.api.IPluginServer;


/**
 * Abstract base class managing additional deployed archives specified in the POM.
 * @author Vincent Michaud
 */
public abstract class AbstractDeploymentCore extends BasicCore implements IDeploymentCore {

    /**
     * Contructor.
     * @param depMojo A deployment mojo
     */
    public AbstractDeploymentCore(final IDeploymentMojo depMojo) throws MojoExecutionException {
        super(depMojo);
    }

   
    /****************************************************/
    /*                    Plugin fields                 */
    /****************************************************/

    /**
     * The deployable manager.
     */
    private DeployableManager manager = null;
    
    /**
     * Configuration of plugin.
     */
    private Configuration config = null;

    /**
     * The behavior when a file msut be deployed / undeployed.
     */
    private IDeploymentBehavior deploymentBehavior = null;

    /**
     * The server.
     */
    private IPluginServer server = null;


    /****************************************************/
    /*          IDeploymentCore implementation          */
    /****************************************************/

    /**
     * Access to the server.
     * @return Instance of the server
     */
    public IPluginServer getServer() {
        return this.server;
    }    

    /**
     * Get the configuration of the goal.
     * @return The configuration, or null if not initialized
     */
    public Configuration getConfiguration() {
        return config;
    }

    /**
     * Get the deployable manager.
     * @return The manager, or null if not initialized
     */
    public DeployableManager getDeployableManager() {
        return this.manager;
    }

    /**
     * Launch the core of the mojo.
     * @throws MojoExecutionException Execution error
     */
    public void start() throws MojoExecutionException {

        // Get the instance of the deployment mojo.
        IDeploymentMojo depMojo = getMojo(IDeploymentMojo.class);

        // Get the configuration of the goal.
        this.config = depMojo.defineConfiguration();

        // Create the deployable manager.
        this.manager = new DeployableManager(getArtifactResolver(), this.config, depMojo.getLocalRepository());

        // Add to the deployable manager additional resources found in the POM.
        this.manager.addResources(depMojo.getAdditionalDeployables());

        // Initialize the server.
        this.server = depMojo.getLaunchedServer(this);

        // Get the deployment behavior.
        this.deploymentBehavior = depMojo.defineDeploymentBehavior(this);

        // Deployment of project.
        manageDeployment();
    }


    /****************************************************/
    /*                 Members functions                */
    /****************************************************/

    
    /**
     * Manage deployment of ressources.
     * @throws MojoExecutionException Execution error
     */
    protected abstract void manageDeployment() throws MojoExecutionException;

    /**
     * Execute deployment action on a resource (deploy / undeploy).
     * @param canonicalPath Canonical path of the deployable
     * @param file File of the deployable
     */
    protected void executeDeploymentAction(final String canonicalPath, final File file) {
        this.deploymentBehavior.deploymentAction(canonicalPath, file);
    }

}
