/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Configuration.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.core;

import java.io.File;
import java.io.FileFilter;
import java.util.LinkedList;
import java.util.List;


/**
 * Internal configuration of a goal.
 * @author Vincent Michaud
 */
public class Configuration {

    /**
     * If true, child of additional directories in the 
     * POM are scanned and only files are extracted.
     */
    private boolean exploreDir;

    /**
     * Define if EAR are explored during the checking of deployable.
     */
    private boolean exploreEAR;

    /**
     * Define if EAR archive are allowed.
     */
    private boolean enableEAR;

    /**
     * Specify Maven project packaging allowed.
     */
    private List<String> allowedPackaging = new LinkedList();

    /**
     * Specify file extension allowed to be deployed.
     */
    private List<String> allowedExtension = new LinkedList();


    /**
     * Constructor.
     * @param additionalDirectoriesExplored  If true, child of additional 
     *        directories in  the POM are scanned and only files are extracted
     * @param exploreEAR Define if EAR are explored during the checking of deployable.
     * @param allowedPackaging Specify Maven project packaging allowed
     * @param allowedExtension Specify file extension allowed to be deployed
     */
    public Configuration(final boolean additionalDirectoriesExplored,
                         final boolean exploreEAR,
                         final String[] allowedPackaging,
                         final String[] allowedExtension) {

        this.exploreDir = additionalDirectoriesExplored;
        this.enableEAR = false;

        if (allowedPackaging != null) {
            for (String str : allowedPackaging) {
                if (str != null && str.length() > 0) {
                    this.allowedPackaging.add(str.toLowerCase());
                }
            }
            this.enableEAR = this.allowedPackaging.contains("ear");
        }

        if (allowedExtension != null) {
            for (String str : allowedExtension) {
                if (str != null && str.length() > 0) {
                    if (str.charAt(0) == '.') {
                        this.allowedExtension.add(str.toLowerCase());
                    } else {
                        this.allowedExtension.add("." + str.toLowerCase());
                    }
                }
            }
            this.enableEAR &= this.allowedExtension.contains(".ear");
        }

        this.exploreEAR = exploreEAR && this.enableEAR;
    }


    /**
     * Determine if packaging is supported by the plugin.
     * @param packaging Packaging of the project
     * @return True if packaging is allowed, false otherwise
     */
    public boolean isAllowedPackaging(final String packaging) {
        String pack = packaging.toLowerCase();
        return this.allowedPackaging.contains(pack);
    }
    
    
    /**
     * Determine if packaging is an EAR and is allowed.
     * @param packaging Packaging of the project
     * @return True if packaging is an EAR and is allowed, false otherwise
     */
    public boolean isAllowedPackagingAndEAR(final String packaging) {
        return (this.enableEAR && packaging.toLowerCase().equals("ear"));
    }


    /**
     * Determine if packaging is allowed and is not an EAR.
     * @param packaging Packaging of the project
     * @return True if packaging is allowed and is not an EAR , false otherwise
     */
    public boolean isAllowedPackagingAndNotEAR(final String packaging) {
        String pack = packaging.toLowerCase();
        return (!pack.equals("ear") && this.allowedPackaging.contains(pack));
    }


    /**
     * Filter for deployable archives.
     */
    private final FileFilter extensionFilter = new FileFilter() {
        public boolean accept(final File file) {
            String name = file.getName().toLowerCase();
            for (String str : allowedExtension) {
                if (name.endsWith(str)) {
                    return true;
                }
            }
            return false;
        }
    };

    /**
     * Get the scanner filter adapted to the configuration.
     * @return Scanner filter
     */
    public FileFilter getExtensionFilter() {
        return this.extensionFilter;
    }

    /**
     * Determine if file extension is allowed.
     * Directories are automatically allowed.
     * @param file File to test
     * @return True if extension is allowed, false otherwise
     */
    public boolean extensionAllowed(final File file) {
        return (this.extensionFilter.accept(file) || file.isDirectory());
    }

    /**
     * Get the list of allowed extension.
     * @return A new list of extensions allowed
     */
    public List<String> getListExtensionAllowed() {
        return new LinkedList(this.allowedExtension);
    }

    /**
     * Determine if directories added by the user are
     * explored or not. If this property is set to false,
     * scanner can dynamically explore directories. If it is
     * set to true, files scanned are those found during the first
     * scanning.
     * @return True if directories are explored, false otherwise.
     */
    public boolean directoriesExplored() {
        return this.exploreDir;
    }

    /**
     * Value of configuration : exploreEAR.
     * @return Boolean
     */
    public boolean exploreEAR() {
        return this.exploreEAR;
    }
}
