/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DeployableManager.java 5740 2011-02-28 15:50:53Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.core;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.model.Dependency;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.ow2.util.maven.plugin.deployment.mapping.Artifact;
import org.ow2.util.maven.plugin.deployment.mapping.Deployables;
import org.ow2.util.maven.plugin.deployment.maven.MavenArtifactResolver;
import org.ow2.util.maven.plugin.deployment.maven.MavenLog;
import org.ow2.util.maven.plugin.deployment.utils.Couple;
import org.ow2.util.maven.plugin.deployment.utils.Scanner;
import org.ow2.util.maven.plugin.deployment.utils.UrlUtils;



/**
 * This class manage all deployables of Maven Java EE projects.
 * It is possible to :
 * - Find easyly and store deployables of Maven projects
 * - List recorded deployables function of custom parameters
 * - Process custom action on all deployables
 * - Manage the multiple location of a Maven artifact :
 *   into a project, or in the local repository.
 * @author Vincent Michaud
 */
public class DeployableManager {

    /****************************************************/
    /*                        Fields                    */
    /****************************************************/


    /**
     * All Maven deployables.
     */
    private Map<String, ProjectDeployable> deployables = new HashMap();

    /**
     * The local repository.
     */
    private ArtifactRepository localRepository;

    /**
     * Configuration of the goal.
     */
    private Configuration config;

    /**
     * Th artifact resolver.
     */
    private MavenArtifactResolver resolver;

    /**
     * Determine if the function addResources(List projects) has been called.
     */
    private boolean addProjectsDone = false;



    /****************************************************/
    /*                      Interfaces                  */
    /****************************************************/


    /**
     * Interface which define action executed on all deployables.
     */
    public interface ProcessAction {

        /**
         * Define action executed on all deployables.
         * @param mainPath Main canonical path of the deployable
         * @param mainFile Main file of the deployable
         * @param secondaryPath Secondary canonical path of the deployable
         * @param secondaryFile Secondary file of the deployable
         */
        void process(final String mainPath, final File mainFile,
                     final String secondaryPath, final File secondaryFile);
    }



    /**
     * Interface which define action executed for list all deployables.
     */
    public interface ProcessListing {

        /**
         * Define action executed on all deployables.
         * @param result Resulting list
         * @param mainPath Main canonical path of the deployable
         * @param mainFile Main file of the deployable
         * @param secondaryPath Secondary canonical path of the deployable
         * @param secondaryFile Secondary file of the deployable
         */
        void process(final List result, final String mainPath, final File mainFile,
                     final String secondaryPath, final File secondaryFile);
    }


    /****************************************************/
    /*                     Functions                    */
    /****************************************************/


    /**
     * Constructor.
     * @param config Configuration of the plugin
     * @param localRepository The local repository
     */
    public DeployableManager(final MavenArtifactResolver resolver, final Configuration config,
                             final ArtifactRepository localRepository) {
        this.config = config;
        this.localRepository = localRepository;
        this.resolver = resolver;
    }

    /**
     * Add a project deployable to the list.
     * If a deployable already exists in the list, the element
     * having priority is the one which has two locations of the
     * deployable. So, the deployable is added function of that.
     * @param dep The deployable
     * @return True if the deployable is added, false otherwise
     */
    private boolean addDeployable(final ProjectDeployable dep) {

        if (!this.deployables.containsKey(dep.mainElmt.first()) || dep.scndElmt != null) {
            File file = dep.mainElmt.second();
            if (this.config.extensionAllowed(file)) {
                this.deployables.put(dep.mainElmt.first(), dep);
                if (dep.scndElmt != null) {
                    this.deployables.put(dep.scndElmt.first(), dep);
                }
                return true;
            } else {
                StringBuffer buffer = new StringBuffer();
                for (String str : this.config.getListExtensionAllowed()) {
                    buffer.append('[' + str + "] ");
                }
                MavenLog.getLog().debug("Can not deploy the file : " + file.getName());
                MavenLog.getLog().debug("Extension of this file is not supported.");
                MavenLog.getLog().debug("Deployed extensions are : " + buffer.toString());
            }
        }
        return false;
    }

    /**
     * Add a file / a directory to the deployable list.
     * If the file is a directory, not any proccess is made on it.
     * @param file The file
     * @return True if the deployable is added, false otherwise
     */
    private boolean addFileResource(final File file) {
        try {
            ProjectDeployable deployable = new ProjectDeployable(file);
            return addDeployable(deployable);
        } catch (IOException ex) {
            MavenLog.getLog().error(file.getName() + " can not be deployed.");
        }
        return false;
    }

    /**
     * Explore a directory and add all its deployables into the list.
     * @param path The path of the directory
     * @return True if all deployabled found are added, false otherwise
     */
    private boolean addDirectoryResources(final String path) {
        boolean result = true;
        List<File> child = Scanner.listFiles(path, true, config.getExtensionFilter());
        if (child != null) {
            for (File file : child) {
                result &= addFileResource(file);
            }
        }
        return result;
    }

    /**
     * Add a file to the deployable list.
     * If the file is a directory and "directoriesExplored" property is
     * enabled in the configuration, all files found in the directory are added.
     * Otherwise, the file / directory is just added.
     * @param file The file
     * @return True if the deployable is added, false otherwise
     */
    public boolean addResource(final File file) {
        if (file != null) {
            if (this.config.directoriesExplored() && file.isDirectory()) {
                return addDirectoryResources(file.getAbsolutePath());
            } else {
                return addFileResource(file);
            }
        }
        return false;
    }

    /**
     * Add an artifact to the deployable list.
     * @param artifact The artifact
     * @throws MojoExecutionException If POM configuration of the goal is not correct
     */
    public boolean addResource(final Artifact artifact) throws MojoExecutionException {
        if (artifact != null) {
            String type = artifact.getType();
            if (type != null && !config.isAllowedPackaging(type)) {
                MavenLog.getLog().error("Artifact type [" + type + "] is not supported.");
            } else {
                if (type == null) {
                    artifact.setType("jar");
                }
                try {
                    ProjectDeployable deployable = new ProjectDeployable(artifact);
                    return addDeployable(deployable);
                } catch (IOException ex) {
                    MavenLog.getLog().error(artifact.getArtifactId() + " can not be deployed.");
                }
            }
        }
        return false;
    }

    /**
     * Find the deployable of a project and add it to the deployable list.
     * If the project is an EAR, the archive will not be explored, even if the
     * mode exploreEAR is actived. The deployable is directly added to the list.
     * @param proj The Maven project
     * @return True if the deployable is added, false otherwise
     * @throws MojoExecutionException If POM configuration of the goal is not correct
     */
    public boolean addResource(final MavenProject proj) throws MojoExecutionException {
        return addResource(proj, Artifact.create(proj));
    }

    /**
     * Find the deployable of a project and add it to the deployable list.
     * If the project is an EAR, the archive will not be explored, even if the
     * mode exploreEAR is actived. The deployable is directly added to the list.
     * @param proj The Maven project
     * @param artifact The artifact corresponding to the project, or null
     * @return True if the deployable is added, false otherwise
     * @throws MojoExecutionException If POM configuration of the goal is not correct
     */
    private boolean addResource(final MavenProject proj, final Artifact artifact) throws MojoExecutionException {
        if (proj != null) {
            String packaging = proj.getPackaging();
            if (!config.isAllowedPackaging(packaging)) {
                MavenLog.getLog().debug("Project packaging [" + packaging + "] is not supported:");
                MavenLog.getLog().debug("     " + proj.getArtifact().toString());
            } else {
                ProjectDeployable deployable = newProjectDeployable(proj, artifact);
                if (deployable != null) {
                    return addDeployable(deployable);
                }
            }
        }
        return false;
    }

    /**
     * Create a new ProjectDeployable.
     * @param proj The Maven project
     * @param artifact The artifact corresponding to the project, or null
     * @return A new ProjectDeployable
     * @throws MojoExecutionException If POM configuration of the goal is not correct
     */
    private ProjectDeployable newProjectDeployable(final MavenProject proj, final Artifact artifact)
            throws MojoExecutionException {
        try {
            return new ProjectDeployable(proj, artifact);
        } catch (IOException ex) {
            MavenLog.getLog().error(proj.getGroupId() + ":" + proj.getArtifactId() + " can not be deployed.");
        }
        return null;
    }


    /**
     * Check if a project is contained in a list of dependencies.
     * @param project The project
     * @param dependencies The list of dependencies
     * @return True if the project is found
     */
    private boolean isDependency(final MavenProject project, final List dependencies) {
        String projArtifactId = project.getArtifactId();
        String projGroupId = project.getGroupId();
        String projVersion = project.getVersion();
        for (Dependency dependency : (List<Dependency>) dependencies) {
            if (projArtifactId.equals(dependency.getArtifactId()) &&
                projGroupId.equals(dependency.getGroupId()) &&
                projVersion.equals(dependency.getVersion())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Determine if a deployable is contained in a compulsary EAR.
     * @param addedEAR All compulsaries EAR
     * @param projDep The deployable
     * @return True if the deployable is contained in a compulsary EAR, false otherwise
     */
    private boolean hasCompulsoriesEAR(final Map<String, ProjectDeployable> addedEAR,
                                       final ProjectDeployable projDep) {
        for (ProjectDeployable earProj : projDep.parents) {
            if (addedEAR.containsKey(earProj.mainElmt.first())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Add all EAR containing a deployable to the compulsaries EAR.
     * @param addedEAR All compulsaries EAR
     * @param projDep The deployable
     */
    private void addCompulsariesEAR(final Map<String, ProjectDeployable> addedEAR,
                                  final ProjectDeployable projDep) {
        for (ProjectDeployable earProj : projDep.parents) {
            addedEAR.put(earProj.mainElmt.first(), earProj);
        }
    }

    /**
     * Check if an deployable is a dependency of another. If true, the deployable can
     * not be deployed alone. The EAR must be deployed. The function add all EAR which
     * must be deployed into a map.
     * @param addedEAR All compulsaries EAR
     * @param proj1 The Maven project for the first deployable
     * @param projDep1 The ProjectDeployable for the first deployable
     * @param proj2 The Maven project for the seconf deployable
     * @param projDep2 The ProjectDeployable for the second deployable
     */
    private void findCompulsoriesEAR(final Map<String, ProjectDeployable> addedEAR,
                                     final MavenProject proj1, final ProjectDeployable projDep1,
                                     final MavenProject proj2, final ProjectDeployable projDep2) {
        if (isDependency(proj1, proj2.getDependencies()) ||
            isDependency(proj2, proj1.getDependencies())) {
            addCompulsariesEAR(addedEAR, projDep1);
            addCompulsariesEAR(addedEAR, projDep2);
        }
    }

    /**
     * Explore a list of projects, and identify all deployables.
     * If the project is an EAR, the archive is explored, only if the
     * exploreEAR mode is actived. In this case, all supported dependencies
     * found in the EAR are added to the list. If the project is an EAR with
     * exploreEAR mode disabled, the resource is directly added to the list.
     * If the project is not an EAR, the resource is added only if it is not
     * contained in an EAR project.
     * @param listProj
     * @throws MojoExecutionException
     */
    public void addResources(final List<MavenProject> listProj) throws MojoExecutionException {
        if (listProj != null && !listProj.isEmpty()) {
            if (addProjectsDone) {
                throw new MojoExecutionException("Function addResources(List<MavenProject>) can be called only "
                                               + "once.\nCollect all your projects before execute this function");
            }

            addProjectsDone = true;
            ArrayList<MavenProject> otherProjects = new ArrayList<MavenProject>();
            Artifact artifact;
            String packaging;

            if (config.exploreEAR()) {
                Map<String, Couple<Artifact, List<ProjectDeployable>>> dependencies = new HashMap();
                Couple<Artifact, List<ProjectDeployable>> dependency;
                List<ProjectDeployable> listPrjDepEAR;
                ArrayList<ProjectDeployable> listPrjDepOther;
                ProjectDeployable prjDepEAR, prjDepOther;
                String artifactKey;

                // Process EAR projects and store other projects into a list.
                for (MavenProject proj : listProj) {

                    packaging = proj.getPackaging();
                    if (config.isAllowedPackagingAndNotEAR(packaging)) {
                        otherProjects.add(proj);

                    } else if (config.isAllowedPackagingAndEAR(packaging)) {
                        prjDepEAR = newProjectDeployable(proj, Artifact.create(proj));

                        // Extract supported dependencies from the EAR project
                        List<Dependency> dependencyList = proj.getDependencies();
                        for (Dependency dep : dependencyList) {
                            if (config.isAllowedPackaging(dep.getType())) {
                                artifact = Artifact.create(dep);
                                artifactKey = artifact.toString();
                                dependency = dependencies.get(artifactKey);

                                if (dependency == null) {
                                    listPrjDepEAR = new LinkedList<ProjectDeployable>();
                                    dependency = new Couple(artifact, listPrjDepEAR);
                                    dependencies.put(artifactKey, dependency);
                                } else {
                                    listPrjDepEAR = dependency.second();
                                }
                                listPrjDepEAR.add(prjDepEAR);
                            }
                        }
                    }
                }

                // Add all deployables not contained in EAR dependencies.
                Iterator<MavenProject> it = otherProjects.iterator();
                listPrjDepOther = new ArrayList<ProjectDeployable>();
                MavenProject project;
                while (it.hasNext()) {
                    project = it.next();
                    artifact = Artifact.create(project);
                    dependency = dependencies.remove(artifact.toString());

                    if (dependency == null) {
                        addResource(project, artifact);
                        it.remove();
                    } else {
                        prjDepOther = newProjectDeployable(project, artifact);
                        prjDepOther.parents = dependency.second();
                        listPrjDepOther.add(prjDepOther);
                    }
                }

                // Add all dependencies not contained in projects.
                for (Couple<Artifact, List<ProjectDeployable>> dep : dependencies.values()) {
                    addResource(dep.first());
                }

                // Now, there are only projects which are dependencies of EAR project.
                int i, j, length = listPrjDepOther.size();
                Map<String, ProjectDeployable> addedEAR = new HashMap();
                for (i = 0; i < length; i++) {
                    prjDepOther = listPrjDepOther.get(i);
                    if (hasCompulsoriesEAR(addedEAR, prjDepOther)) {
                        addCompulsariesEAR(addedEAR, prjDepOther);
                    } else {
                        for (j = i + 1; j < length; j++) {
                            findCompulsoriesEAR(addedEAR, otherProjects.get(i), prjDepOther,
                                                otherProjects.get(j), listPrjDepOther.get(j));
                        }
                    }
                }

                // Add all compulsories EAR.
                for (ProjectDeployable earProj : addedEAR.values()) {
                    addDeployable(earProj);
                }

                // Add all deployable not contained in compulsories EAR.
                for (ProjectDeployable otherProj : listPrjDepOther) {
                    if (!hasCompulsoriesEAR(addedEAR, otherProj)) {
                        addDeployable(otherProj);
                    }
                }
            } else {
                Map<String, Artifact> dependencies = new HashMap();
                Artifact dependency;

                // Process EAR projects and store other projects into a list.
                for (MavenProject proj : listProj) {

                    packaging = proj.getPackaging();
                    if (config.isAllowedPackagingAndNotEAR(packaging)) {
                        otherProjects.add(proj);

                    } else if (config.isAllowedPackagingAndEAR(packaging)) {
                        addResource(proj, Artifact.create(proj));

                        // Extract supported dependencies from the EAR project
                        List<Dependency> dependencyList = proj.getDependencies();
                        for (Dependency dep : dependencyList) {
                            if (config.isAllowedPackaging(dep.getType())) {
                                dependency = Artifact.create(dep);
                                dependencies.put(dependency.toString(), dependency);
                            }
                        }
                    }
                }

                for (MavenProject proj : otherProjects) {
                    artifact = Artifact.create(proj);
                    if (!dependencies.containsKey(artifact.toString())) {
                        addResource(proj, artifact);
                    }
                }
            }
        }
    }


    /**
     * Add deployables to the list.
     * @param dep Deployables to add
     * @return True if all deployables are added, false otherwise
     */
    public boolean addResources(final Deployables dep) throws MojoExecutionException {
        if (dep != null) {
            File file;
            boolean result = true;

            if (dep.getUrls().size() > 0) {
                for (String url : dep.getUrls()) {
                    file = createTempFileFromURL(url);
                    try {
                        MavenLog.getLog().info("Downloading file : " + url);
                        UrlUtils.downloadFile(url, file);
                        result &= addResource(file);
                    } catch (IOException ex) {
                        MavenLog.getLog().error(url + " can not be downloaded.");
                    }
                }
            }

            for (Artifact artifact : dep.getArtifacts()) {
                result &= addResource(artifact);
            }

            for (String path : dep.getFiles()) {
                file = new File(path);
                if (file.isDirectory()) {
                    throw new MojoExecutionException("Bad POM configuration : \"" + file.getName()
                                                   + "\" is not a file but a directory.");
                } else {
                    result &= addFileResource(file);
                }
            }

            if (config.directoriesExplored()) {
                for (String path : dep.getDirectories()) {
                    file = new File(path);
                    if (file.isDirectory()) {
                        addDirectoryResources(path);
                    } else if (file.exists()) {
                        throw new MojoExecutionException("Bad POM configuration : \""
                                  + file.getName() + "\" is not a directory.");
                    }
                }
            } else {
                for (String path : dep.getDirectories()) {
                    file = new File(path);
                    if (!file.exists()) {
                        file.mkdirs();
                    } else if (file.isFile()) {
                        throw new MojoExecutionException("Bad POM configuration : \""
                                  + file.getName() + "\" is not a directory.");
                    }
                    result &= addFileResource(file);
                }
            }
            return result;
        }
        return false;
    }

    /**
     * Get pathes of all deployables recorded.
     * @return A set of path
     */
    public Set<String> listDeployablesPathes() {
        return deployables.keySet();
    }




    /**
     * List all deployables recorded, function of action specified.
     * @param action Action used to select listed deployables
     * @return List containing elements choosen by the action
     */
    public List processListing(final ProcessListing action) {
        List list = new LinkedList();
        processListing(action, list);
        return list;
    }


    /**
     * List all deployables recorded, function of action specified.
     * @param action Action used to select listed deployables
     * @param result Resulting list
     */
    public void processListing(final ProcessListing action, final List result) {
        Set<Entry<String, ProjectDeployable>> es = this.deployables.entrySet();
        Couple<String, File> a, b;
        ProjectDeployable prj;

        for (Entry<String, ProjectDeployable> entry : es) {
            prj = entry.getValue();
            a = prj.mainElmt;
            b = prj.scndElmt;

            if (b == null) {
                action.process(result, a.first(), a.second(), null, null);
            } else if (entry.getKey().equals(a.first())) {
                action.process(result, a.first(), a.second(), b.first(), b.second());
            }
        }
    }


    /**
     * Execute a custom action on all deployables recorded.
     * @param action Action applied on deployables
     */
    public void processAction(final ProcessAction action) {
        Set<Entry<String, ProjectDeployable>> es = this.deployables.entrySet();
        Couple<String, File> a, b;
        ProjectDeployable prj;

        for (Entry<String, ProjectDeployable> entry : es) {
            prj = entry.getValue();
            a = prj.mainElmt;
            b = prj.scndElmt;

            if (b == null) {
                action.process(a.first(), a.second(), null, null);
            } else if (entry.getKey().equals(a.first())) {
                action.process(a.first(), a.second(), b.first(), b.second());
            }
        }
    }



    /**
     * Filter a list functions of recorded deployables and provided configuration.
     * The result list can not have two path representing the same artifact
     * (located in the current project and in the local repository, for example).
     * @param pathList A list of pathes. All multiples pathes are removed from this list.
     */
    public void filterDeployableList(final List<String> pathList) {

        Iterator<String> it = pathList.iterator();
        Set<String> toFilter = new HashSet();
        ProjectDeployable prjDep;
        String path;

        while (it.hasNext()) {
            path = it.next();
            prjDep = this.deployables.get(path);

            if (prjDep != null && prjDep.scndElmt != null) {
                if (path.equals(prjDep.mainElmt.first())) {
                    toFilter.add(prjDep.scndElmt.first());
                } else if (path.equals(prjDep.scndElmt.first()) && toFilter.contains(path)) {
                    it.remove();
                }
            }
        }

        it = pathList.iterator();
        while (it.hasNext()) {
            path = it.next();
            prjDep = this.deployables.get(path);

            if (prjDep != null && prjDep.scndElmt != null) {
                if (path.equals(prjDep.scndElmt.first()) && toFilter.contains(path)) {
                    it.remove();
                }
            }
        }
    }



    /**
     * Create a temp file with the name of URL file.
     * @param url The URL
     * @return A temp file if success, or null otherwise
     */
    private File createTempFileFromURL(final String url) {
        File temp = null;
        if (url != null) {
            int indExt  = url.lastIndexOf(".");
            int indName = url.lastIndexOf("/") + 1;

            if (indName < indExt) {
                String name = url.substring(indName, indExt);
                String ext  = url.substring(indExt);

                if (ext.length() > 1 && name.length() > 0) {
                    try {
                        temp = File.createTempFile(name, ext);
                        temp.deleteOnExit();
                    } catch (IOException ex) {
                        return null;
                    }
                }
            }
        }
        return temp;
    }

    /**
     * Convert the DeployableManager object into a string.
     * @return A string representing the object
     */
    @Override
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        Set<Entry<String, ProjectDeployable>> elmts = this.deployables.entrySet();

        buffer.append("\n\n             ProjectDeployables structure");
        buffer.append("\n##########################################################");
        for (Entry<String, ProjectDeployable> elmt : elmts) {
            buffer.append("\nObject : " + Integer.toHexString(elmt.getValue().hashCode()));
            buffer.append("\nMapKey : " + elmt.getKey());
            buffer.append(elmt.getValue().toString());
            buffer.append("\n##########################################################");
        }
        buffer.append("\n");
        return buffer.toString();
    }



    /**
     * Describe a deployable.
     * A deployable can by described by two elements : one in a project and
     * one in the local repository.
     * Each element is composed to the deployable file and its canonical path.
     * Files used in elements can not exist on the file system.
     */
    protected class ProjectDeployable {

        /**
         * The main element of the deployable.
         * This element can not be null.
         */
        private Couple<String, File> mainElmt;

        /**
         * The secondary element of the deployable.
         * If this element is not null, this element describe the location of
         * the deployable in the local repository (and the main element, the
         * location in the current project).
         * If this element is not null, the deployable has been found in one
         * location only, and this location is described by the main element.
         */
        private Couple<String, File> scndElmt;

        /**
         * List of parent, in the case of deployables contained in EARs.
         */
        private List<ProjectDeployable> parents = null;

        /**
         * Constructor.
         * @param file File to deploy
         * @throws IOException If deployable can not be created
         */
        public ProjectDeployable(final File file) throws IOException {
            initElmts(file, null);
        }

        /**
         * Constructor.
         * @param artifact Artifact to deploy
         * @throws MojoExecutionException If POM configuration of the goal is not correct
         * @throws IOException If deployable can not be created
         */
        public ProjectDeployable(final Artifact artifact) throws MojoExecutionException, IOException {
            File file = artifact.resolve(resolver);
            initElmts(file, null);
        }

        /**
         * Constructor.
         * @param proj Project to deploy
         * @param artifact Artifact to deploy
         * @throws MojoExecutionException If POM configuration of the goal is not correct
         * @throws IOException If deployable can not be created
         */
        public ProjectDeployable(final MavenProject proj, final Artifact artifact)
                throws MojoExecutionException, IOException {
            Couple<File, File> elmts = artifact.resolve(proj, localRepository);
            initElmts(elmts.first(), elmts.second());
        }


        private void initElmts(final File main, final File second) throws IOException {
            if (main == null) {
                this.mainElmt = null;
                throw new IOException();
            } else {
                this.mainElmt = new Couple<String, File>(main.getCanonicalPath(), main);
            }
            if (second == null) {
                this.scndElmt = null;
            } else {
                this.scndElmt = new Couple<String, File>(second.getCanonicalPath(), second);
            }
        }

        /**
         * Convert the ProjectDeployable object into a string.
         * @return A string representing the object
         */
        @Override
        public String toString() {
            StringBuffer buffer = new StringBuffer();
            buffer.append("\n--------------------------------");

            if (this.mainElmt == null) {
                buffer.append("\nElmt 1 : null");
            } else {
                buffer.append("\nFile 1 : ");
                if (this.mainElmt.second() == null) {
                    buffer.append("null");
                } else {
                    try {
                        buffer.append(this.mainElmt.second().getCanonicalPath());
                    } catch (IOException ex) {
                        MavenLog.getLog().error("Error while displaying ProjectDeployable.");
                    }
                }
            }

            if (this.scndElmt == null) {
                buffer.append("\nElmt 2 : null");
            } else {
                buffer.append("\nFile 2 : ");
                if (this.scndElmt.second() == null) {
                    buffer.append("null");
                } else {
                    try {
                        buffer.append(this.scndElmt.second().getCanonicalPath());
                    } catch (IOException ex) {
                        MavenLog.getLog().error("Error while displaying ProjectDeployable.");
                    }
                }
            }

            buffer.append("\n--------------------------------");

            if (this.parents == null) {
                buffer.append("\nParents  : null");
            } else {
                buffer.append("\nParents  :  " + this.parents.size());
                for (ProjectDeployable prjDep : this.parents) {
                    buffer.append("\n    ");
                    buffer.append(Integer.toHexString(prjDep.hashCode()) + "  -  [ ");
                    if (prjDep.mainElmt != null) {
                        if (prjDep.mainElmt.second() != null) {
                            buffer.append(prjDep.mainElmt.second().getName() + " ]");
                        } else if (prjDep.mainElmt.first() != null) {
                            buffer.append(prjDep.mainElmt.first() + " ]");
                        } else {
                            buffer.append("null ]");
                        }
                    } else {
                        buffer.append("null ]");
                    }
                }
            }

            return buffer.toString();
        }

    }

}





