/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: UnboundDeploymentCore.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.core;

import java.io.File;
import java.util.LinkedList;
import java.util.List;
import org.apache.maven.plugin.MojoExecutionException;
import org.ow2.util.maven.plugin.deployment.core.DeployableManager.ProcessAction;
import org.ow2.util.maven.plugin.deployment.api.IDeploymentUnboundMojo;
import org.ow2.util.maven.plugin.deployment.api.IPluginServer;
import org.ow2.util.maven.plugin.deployment.maven.MavenLog;
import org.ow2.util.maven.plugin.deployment.utils.ConsoleScanner;
import org.ow2.util.maven.plugin.deployment.utils.RepeatedTask.EventAction;
import org.ow2.util.maven.plugin.deployment.utils.Scanner;
import org.ow2.util.maven.plugin.deployment.utils.Scanner.Listener;


/**
 * Abstract mojo providing hot deployment of project resources.
 * Resources are sought in the current project and in the children projects
 * and are automaticaly (re)deployed.
 * This mojo loop in order to detect modifications of ressources.
 * All inherited mojo must be unbound to the project lifecycle.
 * @author Vincent Michaud
 */
public class UnboundDeploymentCore extends AbstractDeploymentCore implements ProcessAction {

    /**
     * Contructor.
     * @param depMojo A deployment mojo
     */
    public UnboundDeploymentCore(final IDeploymentUnboundMojo depMojo) throws MojoExecutionException {
        super(depMojo);
    }


    /****************************************************/
    /*                    Plugin fields                 */
    /****************************************************/

    /**
     * The scanner file.
     */
    private Scanner scanner = null;

    /**
     * List storing file to deploy / redeploy.
     */
    private final List<String> deploymentList = new LinkedList<String>();

    /**
     * Wait time when the instance of server is local.
     */
    private static final int LOCAL_WAIT_TIME = 100;

    /**
     * Wait time when the instance of server is remote.
     */
    private static final int REMOTE_WAIT_TIME = 5000;


    /****************************************************/
    /*                 Members functions                */
    /****************************************************/

    /**
     * Synchronisation with the server.
     * This function waits the server stop.
     */
    protected void waitServerStopped() {

        int wait = LOCAL_WAIT_TIME;
        IPluginServer server = getServer();
        boolean isRemote = server.isRemoteInstance();
        if (isRemote) {
            wait = REMOTE_WAIT_TIME;
        }

        try {
            while (!server.isStopped()) {
                try {
                    Thread.sleep(wait);
                } catch (InterruptedException ex) {
                    MavenLog.getLog().debug("Sleep interrupted.");
                }
            }
        } catch (Exception ex) {
            if (isRemote) {
                MavenLog.getLog().info("The remote server has been stopped.");
            }
        }
    }

    /**
     * Add a file or a directory to scan.
     * @param path The path of the file
     * @param file The file
     */
    protected void addScannedFile(final String path, final File file) {
        if (file.isDirectory()) {
            this.scanner.addScanDirectory(path, scannerListener, true, false,
                                          getConfiguration().getExtensionFilter());
        } else {
            this.scanner.addScanFile(path, scannerListener);
        }
        MavenLog.getLog().debug("File added to the scanner : " + path);
    }

    /**
     * Allow to deploy only existing files.
     * @param mainPath Main canonical path of the deployable
     * @param mainFile Main file of the deployable
     * @param secondaryPath Secondary canonical path of the deployable
     * @param secondaryFile Secondary file of the deployable
     */
    public void process(final String mainPath, final File mainFile,
                        final String secondaryPath, final File secondaryFile) {
        addScannedFile(mainPath, mainFile);
        if (secondaryFile != null && secondaryFile.exists() && !mainFile.exists()) {
            addScannedFile(secondaryPath, secondaryFile);
        }
    }

    /**
     * Check the project, deploy found archives and enable the scanner if necessary.
     * This function must be executed when the goal run is not bound to
     * the build lifecycle.
     * @throws MojoExecutionException If configuration of plugin is not correct
     */
    protected void manageDeployment() throws MojoExecutionException {

        // Get the instance of the deployment mojo.
        IDeploymentUnboundMojo depMojo = getMojo(IDeploymentUnboundMojo.class);

        // Get the deployable manager.
        DeployableManager manager = getDeployableManager();

        // Add all resources contained in the reactor project in the manager.
        manager.addResources(depMojo.getReactorProjects());

        // Initialize the scanner.
        long scanInterval = depMojo.getScanInterval();
        this.scanner = new Scanner();
        this.scanner.setListenersInheritanceMode(Scanner.NORMAL_INHERITANCE_MODE);
        this.scanner.setInterval(scanInterval);
        this.scanner.setReportFilesOnStartup(true);
        this.scanner.setActionAfterScan(this.processResult);

        // Add all deployables to scan.
        manager.processAction(this);

        // Start the scanner.
        if (depMojo.isAutoDeploymentEnabled()) {
            MavenLog.getLog().info("Automatic deployment is enabled with a scan delay of " + scanInterval + " ms.");
            this.scanner.start();

        } else {
            this.scanner.launchNow();
            MavenLog.getLog().info("Automatic deployement is disabled. Press [ENTER] for redeploy archives.");
            ConsoleScanner consoleScanner = new ConsoleScanner(launchScan);
            consoleScanner.start();
        }
        waitServerStopped();
    }


    /**
     * A listener for the scanner which store added files and modified
     * files in the same list, in order to process after.
     */
    private final Listener scannerListener = new Listener() {
        public void reportAction(final String canonicalPath, final int typeAction) {
            switch (typeAction) {
                case Scanner.DELETED_ACTION: break;
                default: deploymentList.add(canonicalPath); break;
            }
        }
    };

    /**
     * Process the result of the scan, stored in the deployment list.
     */
    private EventAction processResult = new EventAction() {
        public void process() {
            getDeployableManager().filterDeployableList(deploymentList);
            IPluginServer server = getServer();
            for (String path : deploymentList) {
                MavenLog.getLog().info("Deploying file : " + path);
                server.deployArchive(path);
            }
            deploymentList.clear();
        }
    };

    /**
     * Action for the ConsoleScanner.
     * Trigger the scan of deployed files.
     */
    private final ConsoleScanner.Action launchScan = new ConsoleScanner.Action() {
        public void execute() {
            scanner.launchNow();
            if (!scanner.hasChanged()) {
                MavenLog.getLog().info("Deployment : No changes detected.");
            }
        }
    };

}
