/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Artifact.java 5740 2011-02-28 15:50:53Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.mapping;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.model.Dependency;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.ow2.util.maven.plugin.deployment.maven.MavenArtifactResolver;
import org.ow2.util.maven.plugin.deployment.utils.Couple;

/**
 * This class manipule deployable artifacts. This class is mapped by Maven with
 * the POM configuration file.
 * @author Alexandre Deneux
 * @author Bilel Souadnia
 * @author Vincent Michaud
 */
public class Artifact {

    /**
     * Artifact Group Id.
     */
    private String groupId;

    /**
     * Artifact Name.
     */
    private String artifactId;

    /**
     * Artifact Version.
     */
    private String version;

    /**
     * Type of artifact.
     */
    private String type;

    /**
     * Create an artifact.
     * @param groupId Group ID
     * @param artifactId Artifact ID
     * @param version Version
     * @param type Type of packaging
     * @return New artifact
     */
    public static Artifact create(final String groupId, final String artifactId, final String version, final String type) {
        Artifact artifact = new Artifact();
        artifact.groupId = groupId;
        artifact.artifactId = artifactId;
        artifact.version = version;
        artifact.type = type;
        return artifact;
    }

    /**
     * Create an artifact.
     * @param project Project representing the artifact.
     * @return New artifact
     */
    public static Artifact create(final MavenProject project) {
        Artifact artifact = new Artifact();
        artifact.groupId = project.getGroupId();
        artifact.artifactId = project.getArtifactId();
        artifact.version = project.getVersion();
        artifact.type = project.getPackaging();
        return artifact;
    }

    /**
     * Create an artifact.
     * @param dependency Dependency representing the artifact.
     * @return New artifact
     */
    public static Artifact create(final Dependency dependency) {
        Artifact artifact = new Artifact();
        artifact.groupId = dependency.getGroupId();
        artifact.artifactId = dependency.getArtifactId();
        artifact.version = dependency.getVersion();
        artifact.type = dependency.getType();
        return artifact;
    }

    /**
     * Get the artifactId.
     * @return Returns the artifactId, or null if not specified
     */
    public String getArtifactId() {
        if (this.artifactId.length() == 0) {
            return null;
        }
        return this.artifactId;
    }

    /**
     * Set the artifactId.
     * @param artifactId The artifactId to set
     */
    public void setArtifactId(final String artifactId) {
        this.artifactId = artifactId;
    }

    /**
     * Get the groupId.
     * @return Returns the groupId, or null if not specified
     */
    public String getGroupId() {
        if (this.groupId.length() == 0) {
            return null;
        }
        return this.groupId;
    }

    /**
     * Set the groupId.
     * @param groupId The groupId to set
     */
    public void setGroupId(final String groupId) {
        this.groupId = groupId;
    }

    /**
     * Get the type.
     * @return Returns the type, or null if not specified
     */
    public String getType() {
        if (this.type == null || this.type.length() == 0) {
            return null;
        }
        return this.type;
    }

    /**
     * Set the type.
     * @param type The type to set
     */
    public void setType(final String type) {
        this.type = type;
    }

    /**
     * Get the version.
     * @return Returns the version, or null if not specified
     */
    public String getVersion() {
        if (this.version.length() == 0) {
            return null;
        }
        return this.version;
    }

    /**
     * Set the version.
     * @param version The version to set.
     */
    public void setVersion(final String version) {
        this.version = version;
    }

    /**
     * Get extension of the artifact.
     * @throws MojoExecutionException If type is missing
     * @return String representing extension of artifact
     */
    public String getExtension() throws MojoExecutionException {
        if (isEmpty(this.type)) {
            throw new MojoExecutionException("Bad POM configuration : Type of artifact is missing.");
        }
        return getExtension(this.type);
    }

    /**
     * Get name of file described by the artifact.
     * @throws MojoExecutionException If version of ID are missing
     * @return File name of artifact
     */
    public String getFilename() throws MojoExecutionException {
        if (isEmpty(this.artifactId)) {
            throw new MojoExecutionException("Bad POM configuration : ArtifactId is missing.");
        } else if (isEmpty(this.version)) {
            throw new MojoExecutionException("Bad POM configuration : Version of artifact is missing.");
        }
        return getFilename(this.artifactId, this.version, getExtension());
    }

    /**
     * Get extension of the artifact.
     * @param type The type of the artifact
     * @return String representing extension of artifact
     */
    private static String getExtension(final String type) {
        if (type.equals("ejb") || type.equals("ejb3")) {
            return new String("jar");
        }
        return new String(type);
    }

    /**
     * Create an artifact.
     * @param artifactId Artifact ID
     * @param version Version
     * @param ext Extension of the file
     * @return File name of artifact
     */
    private static String getFilename(final String artifactId, final String version, final String ext) {
        return new String(artifactId + "-" + version + "." + ext);
    }

    /**
     * Find the location of artifact in the local repository.
     * @param filename Name of the file to found
     * @param localRepository The local repository
     * @return File described by the artifact
     * @throws MojoExecutionException If a required parameter is missing
     */
    private File checkRepository(final String filename, final ArtifactRepository localRepository)
            throws MojoExecutionException {

        if (localRepository == null) {
            throw new MojoExecutionException("Local repository is not defined.");
        }

        String dirPath = getFilePath(this.groupId, this.artifactId, this.version, localRepository);
        return new File(dirPath, filename);
    }

    /**
     * Get the path directory of the artifact.
     * @param groupId Group ID
     * @param artifactId Artifact ID
     * @param version Version
     * @param localRepository
     * @return The path
     */
    private static String getFilePath(final String groupId, final String artifactId, final String version,
                                      final ArtifactRepository localRepository) {

        StringBuilder builder = new StringBuilder(localRepository.getBasedir());
        char last = builder.charAt(builder.length() - 1);
        if (last != '/' && last != '\\') {
            builder.append(File.separatorChar);
        }
        builder.append(groupId.replace('.', File.separatorChar));
        builder.append(File.separatorChar);
        builder.append(artifactId);
        builder.append(File.separatorChar);
        builder.append(version);
        return builder.toString();
    }

    /**
     * Determine if an artifact is associated to a project.
     * @param project The project
     * @return True if artifact is associated to the project, false otherwise
     */
    public boolean isProject(final MavenProject project) {
        return (project.getArtifactId().equals(this.artifactId)
             && project.getGroupId().equals(this.groupId)
             && project.getVersion().equals(this.version)
             && project.getPackaging().equals(this.type));
    }

    /**
     * Determine if the artifact is one of specified projects.
     * @param projects Projects checked
     * @return The project representing the artifact, or null nothing is found
     */
    public MavenProject isProject(final List<MavenProject> projects) {
        for (MavenProject project : projects) {
            if (isProject(project)) {
                return project;
            }
        }
        return null;
    }

    /**
     * Find locations of an artifact : in projects and/or in the local repository.
     * @param project Project representing the artifact. This parameter can be null
     * @param localRepository The local repository
     * @return Couple of elements described by the artifact. If project is not specified,
     *         first element of the couple is the couple path/file located in the reporitory.
     *         The second element is null. Otherwise, first element is the couple path/file
     *         located in the project and the second is in the local repository.
     *         Files can not exist on the file system.
     * @throws MojoExecutionException If a required parameter is missing
     * @throws IOException If an I/O occurs
     */
    public Couple<File, File> resolve(final MavenProject project, final ArtifactRepository localRepository)
            throws MojoExecutionException, IOException {

        Couple<File, File> result = new Couple<File, File>();
        String filename = getFilename();

        if (isEmpty(this.groupId)) {
            throw new MojoExecutionException("Bad POM configuration : groupId is missing.");
        }

        File file = checkRepository(filename, localRepository);

        if (project != null) {
            result.setSecond(file);
            file = new File(project.getBuild().getDirectory(), filename);
        }

        result.setFirst(file);
        return result;
    }

    /**
     * Resolve the location of the artifact in the local repository.
     * @param groupId Group ID
     * @param artifactId Artifact ID
     * @param version Version
     * @param type Type of artifact
     * @param localRepository The local repository
     * @return The file of the artifact. The file can not exist on the file system
     */
    public static File resolve(final String groupId, final String artifactId, final String version,
                               final String type, final ArtifactRepository localRepository) {
        if (!isEmpty(groupId) && !isEmpty(artifactId) && !isEmpty(version) && !isEmpty(type)) {
            String filename = getFilename(artifactId, version, getExtension(type));
            String dirPath = getFilePath(groupId, artifactId, version, localRepository);
            return new File(dirPath, filename);
        }
        return null;
    }

    /**
     * Resolve the artifact. If the artifact is not found in the local repository,
     * remotes repositories are checked. If the artifact is found in one of remote
     * repositories, the artifact is downloaded and installed into the local repository.
     * @param resolver An artifact resolver
     * @return The resolved file, or null if nothing is found
     * @throws MojoExecutionException If a required parameter is missing
     */
    public File resolve(final MavenArtifactResolver resolver) throws MojoExecutionException {
        if (isEmpty(this.groupId)) {
            throw new MojoExecutionException("Bad POM configuration : groupId is missing.");
        } else if (isEmpty(this.artifactId)) {
            throw new MojoExecutionException("Bad POM configuration : artifactId is missing.");
        } else if (isEmpty(this.type)) {
            throw new MojoExecutionException("Bad POM configuration : Type of artifact is missing.");
        } else if (isEmpty(this.version)) {
            throw new MojoExecutionException("Bad POM configuration : Version of artifact is missing.");
        }
        return resolver.resolveFile(this.groupId, this.artifactId, this.version, this.type, null, null);
    }

    /**
     * Convert the artifact into a string.
     * @return A String representing the artifact
     */
    @Override
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        if (this.groupId != null) {
            buffer.append(this.groupId);
        }
        buffer.append(':');
        if (this.artifactId != null) {
            buffer.append(this.artifactId);
        }
        buffer.append(':');
        if (this.type != null) {
            buffer.append(this.type);
        }
        buffer.append(':');
        if (this.version != null) {
            buffer.append(this.version);
        }
        return buffer.toString();
    }

    /**
     * Determine if string is empty or not.
     * @param str A string
     * @return True if string is empty, false otherwise
     */
    private static boolean isEmpty(final String str) {
        return (str == null || str.length() == 0);
    }

}
