/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MavenArtifactResolver.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.maven;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.factory.ArtifactFactory;
import org.apache.maven.artifact.metadata.ArtifactMetadataRetrievalException;
import org.apache.maven.artifact.metadata.ArtifactMetadataSource;
import org.apache.maven.artifact.metadata.ResolutionGroup;
import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.resolver.ArtifactNotFoundException;
import org.apache.maven.artifact.resolver.ArtifactResolutionException;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.artifact.resolver.ArtifactResolver;
import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
import org.apache.maven.artifact.versioning.VersionRange;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectBuilder;
import org.apache.maven.project.ProjectBuildingException;


/**
 * Class for resolving Maven artifact location.
 * @author Vincent Michaud
 */
public final class MavenArtifactResolver {

    /**
     * Remotes repositories of the current executed project.
     */
    private List<ArtifactRepository> remoteRepositories;

    /**
     * The local repository.
     */
    private ArtifactRepository localRepository;

    /**
     * An artifact resolver.
     */
    private ArtifactResolver resolver;

    /**
     * An artifact factory.
     */
    private ArtifactFactory factory;

    /**
     * An artifact metadata source.
     */
    private ArtifactMetadataSource metadataSource;

    /**
     * A Maven project builder.
     */
    private MavenProjectBuilder projectBuilder;

    /**
     * The project of the current plugin.
     */
    private MavenProject pluginProject;

    /**
     * The project of the current plugin.
     */
    private Artifact pluginArtifact;

    /**
     * The project of the current plugin.
     */
    private List<ArtifactRepository> pluginRepositories;


    /**
     * Constructor.
     * @param project Current executed project
     * @param localRepository The local repository
     * @param resolver An artifact resolver
     * @param factory An artifact factory
     * @param metadataSource An artifact metagata source. Can be null if the function
     *                       resolveTransitively() is not used.
     * @param projectBuilder A Maven project builder. Can be null if the function
     *                       resolveTransitively() is not used.
     * @param pluginGroupId Group of the plugin
     * @param pluginArtifactId ID of the plugin
     * @param pluginVersion Version of the plugin
     * @throws MojoExecutionException If the version of the plugin is invalid or the
     *                                plugin project can not be created
     */
    public MavenArtifactResolver(final MavenProject project, final ArtifactRepository localRepository,
                                 final ArtifactResolver resolver, final ArtifactMetadataSource metadataSource,
                                 final ArtifactFactory factory, final MavenProjectBuilder projectBuilder,
                                 final String pluginGroupId, final String pluginArtifactId, 
                                 final String pluginVersion) throws MojoExecutionException {
        try {
            this.localRepository = localRepository;
            this.remoteRepositories = project.getRemoteArtifactRepositories();
            this.factory = factory;
            this.resolver = resolver;
            this.metadataSource = metadataSource;
            this.projectBuilder = projectBuilder;
            VersionRange vr = VersionRange.createFromVersionSpec(pluginVersion);
            this.pluginArtifact = factory.createPluginArtifact(pluginGroupId, pluginArtifactId, vr);
            this.pluginProject = projectBuilder.buildFromRepository(this.pluginArtifact, null, localRepository);
            this.pluginRepositories = pluginProject.getRemoteArtifactRepositories();
        } catch (Exception ex) {
            throw new MojoExecutionException("The MavenArtifactResolver can not be created.", ex);
        }
    }

    /**
     * Get repositories of the current plugin.
     * @return A list of repositories
     */
    public List<ArtifactRepository> getPluginRepositories() {
        return new LinkedList<ArtifactRepository>(this.pluginRepositories);
    }

    /**
     * Resolve a Maven artifact. If the artifact is not found in the local repository,
     * remotes repositories are checked. If the artifact is found in one of remote
     * repositories, the artifact is downloaded and installed into the local repository.
     * @param groupId Group of the artifact
     * @param artifactId ID of the artifact
     * @param version Version of the artifact
     * @param type Type of the artifact
     * @param classifier The classifier of the artifact
     * @param additionalRepositories Other repositories checked. Can be null
     * @return The Maven artifact, or null if an error occurs
     */
    public Artifact resolve(final String groupId, final String artifactId, final String version,
                            final String type, final String classifier,
                            final List<ArtifactRepository> additionalRepositories) {
        if (groupId != null && artifactId != null && version != null && type != null
         && groupId.length() != 0 && artifactId.length() != 0 && version.length() != 0 && type.length() != 0) {

            Artifact artifact = null;

            try {
                // Create the list of repositories.
                List<ArtifactRepository> repositories = new ArrayList<ArtifactRepository>();
                Set<String> containedRepositories = new HashSet<String>();
                addRepositories(repositories, containedRepositories, this.remoteRepositories);
                addRepositories(repositories, containedRepositories, additionalRepositories);

                // Create the artifact.
                VersionRange versionSpec = VersionRange.createFromVersionSpec(version);
                artifact = factory.createDependencyArtifact(groupId, artifactId, versionSpec, type, classifier, null);

                // Resolve the artifact.
                ResolutionGroup resolutionGroup = this.metadataSource.retrieve(artifact, this.localRepository, repositories);
                addRepositories(repositories, containedRepositories, resolutionGroup.getResolutionRepositories());
                this.resolver.resolve(artifact, repositories, this.localRepository);
                return artifact;
                
            } catch (ArtifactMetadataRetrievalException ex) {
                MavenLog.getLog().error("Unable to download metadata from repository for artifact : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            } catch (InvalidVersionSpecificationException ex) {
                MavenLog.getLog().error("Invalid version for the artifact : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            } catch (ArtifactResolutionException ex) {
                MavenLog.getLog().error("Artifact can not be resolved : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            } catch (ArtifactNotFoundException ex) {
                MavenLog.getLog().error("Artifact not found : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            }
        } else {
            MavenLog.getLog().error("Unable to resolve artifact : Bad parameters.");
        }
        return null;
    }

    /**
     * Resolve a Maven artifact. If the artifact is not found in the local repository,
     * remotes repositories are checked. If the artifact is found in one of remote
     * repositories, the artifact is downloaded and installed into the local repository.
     * @param groupId Group of the artifact
     * @param artifactId ID of the artifact
     * @param version Version of the artifact
     * @param type Type of the artifact
     * @param classifier The classifier of the artifact
     * @param additionalRepositories Other repositories checked. Can be null
     * @return The file described by the artifact, or null if an error occurs
     */
    public File resolveFile(final String groupId, final String artifactId, final String version,
                            final String type, final String classifier,
                            final List<ArtifactRepository> additionalRepositories) {
        Artifact artifact = resolve(groupId, artifactId, version, type, classifier, additionalRepositories);
        if (artifact != null) {
            return artifact.getFile();
        }
        return null;
    }


    /**
     * Add repositories into a result list. Repositories are filtered function
     * of already contained repositories.
     * @param result The resulting list
     * @param urlsContained Set of contained repositories in the resulting list
     * @param repositoryToAdd List of repositories to add. Can be null
     */
    private static void addRepositories(final List<ArtifactRepository> result, final Set<String> urlsContained,
                                        final List<ArtifactRepository> repositoryToAdd) {
        if (repositoryToAdd != null) {
            String urlRepository = null;
            for (ArtifactRepository ar : repositoryToAdd) {
                urlRepository = ar.getUrl();
                if (!urlsContained.contains(urlRepository)) {
                    result.add(ar);
                    urlsContained.add(urlRepository);
                }
            }
        }
    }

    /**
     * Resolve transitively a Maven artifact. If the artifact is not found in the local repository,
     * remotes repositories are checked. If the artifact is found in one of remote
     * repositories, the artifact is downloaded and installed into the local repository.
     * @param groupId Group of the artifact
     * @param artifactId ID of the artifact
     * @param version Version of the artifact
     * @param type Type of the artifact
     * @param classifier The classifier of the artifact
     * @param additionalRepositories Other repositories checked. Can be null
     * @return A set of resolved Maven artifacts, or null if an error occurs
     */
    public Set<Artifact> resolveTransitively(final String groupId, final String artifactId,
                                             final String version, final String type, final String classifier,
                                             final List<ArtifactRepository> additionalRepositories) {

        if (groupId != null && artifactId != null && version != null && type != null
         && groupId.length() != 0 && artifactId.length() != 0 && version.length() != 0 && type.length() != 0) {
            try {
                // Create the list of repositories.
                Artifact pomArtifact;
                Set<Artifact> dependencies;
                List<ArtifactRepository> repositories = new LinkedList<ArtifactRepository>();
                Set<String> containedRepositories = new HashSet<String>();
                addRepositories(repositories, containedRepositories, this.remoteRepositories);
                addRepositories(repositories, containedRepositories, additionalRepositories);

                // Create the artifact.
                VersionRange vr = VersionRange.createFromVersionSpec(version);
                Artifact artifact = this.factory.createDependencyArtifact(groupId, artifactId, vr, type, classifier, null);

                // Retrieve metadata from the artifact.
                ResolutionGroup resolutionGroup = this.metadataSource.retrieve(artifact, this.localRepository, repositories);
                repositories = resolutionGroup.getResolutionRepositories();

                // Create the list of dependencies
                if (type.equals("pom")) {
                    dependencies = resolutionGroup.getArtifacts();
                    dependencies.add(artifact);                
                    MavenProject prj = this.projectBuilder.buildFromRepository(artifact, repositories, this.localRepository);
                    pomArtifact = prj.getArtifact();
                } else {
                    dependencies = new HashSet<Artifact>();
                    dependencies.add(artifact);
                    pomArtifact = this.pluginArtifact;
                }

                // Resolve artifacts.
                ArtifactResolutionResult resolution = this.resolver.resolveTransitively(
                                                            dependencies, pomArtifact, Collections.EMPTY_MAP,
                                                            this.localRepository, repositories,
                                                            this.metadataSource, null, Collections.EMPTY_LIST);

                Set<Artifact> result = resolution.getArtifacts();
                if (result == null || result.isEmpty()) {
                    throw new ArtifactNotFoundException("Artifact not found.", artifact);
                }
                return result;

            } catch (ArtifactMetadataRetrievalException ex) {
                MavenLog.getLog().error("Unable to download metadata from repository for artifact : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            } catch (ProjectBuildingException ex) {
                MavenLog.getLog().error("Can not build the project : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            } catch (InvalidVersionSpecificationException ex) {
                MavenLog.getLog().error("Invalid version specification for the artifact : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            } catch (ArtifactResolutionException ex) {
                MavenLog.getLog().error("Artifact can not be resolved : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            } catch (ArtifactNotFoundException ex) {
                MavenLog.getLog().error("Artifact not found : " + groupId
                                      + ":" + artifactId + ":" + version + ":" + type);
            }
        } else {
            MavenLog.getLog().error("Unable to resolve artifact : Bad parameters.");
        }
        return null;
    }

    /**
     * Resolve transitively a Maven artifact. If the artifact is not found in the local repository,
     * remotes repositories are checked. If the artifact is found in one of remote
     * repositories, the artifact is downloaded and installed into the local repository.
     * @param groupId Group of the artifact
     * @param artifactId ID of the artifact
     * @param version Version of the artifact
     * @param type Type of the artifact
     * @param classifier The classifier of the artifact
     * @param additionalRepositories Other repositories checked. Can be null
     * @return A list of resolved files described by the artifact, or null if an error occurs
     */
    public List<File> resolveTransitivelyFiles(final String groupId, final String artifactId,
                                               final String version, final String type, final String classifier,
                                               final List<ArtifactRepository> additionalRepositories) {
        Set<Artifact> artifacts = resolveTransitively(groupId, artifactId, version, type, 
                                                      classifier, additionalRepositories);
        if (artifacts != null) {
            List<File> result = new ArrayList<File>();
            for (Artifact artifact : artifacts) {
                result.add(artifact.getFile());
            }
            return result;
        }
        return null;
    }

    /**
     * Resolve transitively a Maven artifact. If the artifact is not found in the local repository,
     * remotes repositories are checked. If the artifact is found in one of remote
     * repositories, the artifact is downloaded and installed into the local repository.
     * @param result A result list. A file is added in the list only if there is not another file
     *               with the same name.
     * @param groupId Group of the artifact
     * @param artifactId ID of the artifact
     * @param version Version of the artifact
     * @param type Type of the artifact
     * @param classifier The classifier of the artifact
     * @param additionalRepositories Other repositories checked. Can be null
     * @return True if artifact is resolved, false if an error occurs
     */
    public boolean resolveTransitivelyFiles(final List<File> result, final String groupId, final String artifactId,
                                            final String version, final String type, final String classifier,
                                            final List<ArtifactRepository> additionalRepositories) {
        Set<Artifact> artifacts = resolveTransitively(groupId, artifactId, version, type, 
                                                      classifier, additionalRepositories);
        if (artifacts != null) {
            boolean found;
            String name = null;
            File file = null;
            for (Artifact artifact : artifacts) {
                file = artifact.getFile();
                name = file.getName();
                found = false;

                for (File f : result) {
                    if (f.getName().equals(name)) {
                        found = true;
                        break;
                    }
                }
                if (!found) {
                    result.add(file);
                }
            }
            return true;
        }
        return false;
    }

    /**
     * Get a Maven project.
     * @param remoteRepositories List of remote repository
     * @param localRepository The local repository
     * @param factory An artifact factory
     * @param projectBuilder A Maven project builder.
     * @param groupId Group of the artifact
     * @param artifactId ID of the artifact
     * @param version Version of the artifact
     * @param type Type of the artifact
     * @return The project
     */
    public static MavenProject getProject(final List<ArtifactRepository> remoteRepositories,
                                          final ArtifactRepository localRepository,
                                          final ArtifactFactory factory,
                                          final MavenProjectBuilder projectBuilder,
                                          final String groupId, final String artifactId,
                                          final String version, final String type) {
        try {
            Artifact artifact = factory.createBuildArtifact(groupId, artifactId, version, type);
            return projectBuilder.buildFromRepository(artifact, remoteRepositories, localRepository);
        } catch (ProjectBuildingException ex) {
            MavenLog.getLog().error("Can not build the project : " + groupId
                                  + ":" + artifactId + ":" + version + ":" + type);
        }
        return null;
    }

    /**
     * Get a Maven project.
     * @param localRepository The local repository
     * @param factory An artifact factory
     * @param projectBuilder A Maven project builder.
     * @param groupId Group of the artifact
     * @param artifactId ID of the artifact
     * @param version Version of the artifact
     * @param type Type of the artifact
     * @return The project
     */
    public static MavenProject getLocalProject(final ArtifactRepository localRepository,
                                               final ArtifactFactory factory,
                                               final MavenProjectBuilder projectBuilder,
                                               final String groupId, final String artifactId,
                                               final String version, final String type) {
        try {
            Artifact artifact = factory.createBuildArtifact(groupId, artifactId, version, type);
            return projectBuilder.buildFromRepository(artifact, null, localRepository);
        } catch (ProjectBuildingException ex) {
            MavenLog.getLog().error("Can not build the project : " + groupId
                                  + ":" + artifactId + ":" + version + ":" + type);
        }
        return null;
    }

}
