/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MavenPluginClassloader.java 5739 2011-02-28 15:50:18Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.maven;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.List;

import org.codehaus.plexus.PlexusConstants;
import org.codehaus.plexus.PlexusContainer;
import org.codehaus.plexus.PlexusContainerException;
import org.codehaus.plexus.context.Context;
import org.codehaus.plexus.context.ContextException;


/**
 * A class for extending the plugin classloader.
 * @author Vincent Michaud
 */
public final class MavenPluginClassloader {

    /**
     * The plexus container of the plugin.
     */
    private PlexusContainer pluginContainer = null;

    /**
     * The realm of the plugin. (Object as with maven 2.x and 3.x the package of the realm class is different)
     */
    private Object pluginRealm = null;

    /**
     * The instance of the plugin classloader.
     */
    private static MavenPluginClassloader instance = new MavenPluginClassloader();

    /**
     * Constructor.
     */
    private MavenPluginClassloader() {
    }

    /**
     * Get the instance of the plugin classloader.
     * @param context The Plexus context, provided by Maven
     * @return The instance of the plugin classloader
     */
    public static MavenPluginClassloader getInstance(final Context context) {
        if (instance.pluginContainer == null) {

            PlexusContainer container;
            try {
                container = (PlexusContainer) context.get(PlexusConstants.PLEXUS_KEY);
            } catch (ContextException e) {
                MavenLog.getLog().error("Error while initializing the MavenPluginClassloader.", e);
                throw new RuntimeException("Error while initializing the MavenPluginClassloader.", e);
            }
            if (container != null) {
                instance.pluginContainer = container;

                try {
                    final Method getContainerRealmMethod = container.getClass().getMethod("getContainerRealm");
                    instance.pluginRealm = getContainerRealmMethod.invoke(container);
                } catch (final NoSuchMethodException e) {
                    MavenLog.getLog().error("Error while initializing the MavenPluginClassloader.", e);
                    throw new RuntimeException(e);
                } catch (final IllegalAccessException e) {
                    MavenLog.getLog().error("Error while initializing the MavenPluginClassloader.", e);
                    throw new RuntimeException(e);
                } catch (final InvocationTargetException e) {
                    MavenLog.getLog().error("Error while initializing the MavenPluginClassloader.", e);
                    throw new RuntimeException(e.getCause());
                }
            }

        }
        return instance;
    }

    /**
     * Get the instance of the plugin classloader.
     * @return The instance of the plugin classloader
     */
    public static MavenPluginClassloader getInstance() {
        return instance;
    }

    /**
     * Report an error.
     * @param resources A list
     */
    private static void reportListError(final List<?> resources) {
        MavenLog.getLog().error("The MavenPluginClassloader is not initialized.");
        MavenLog.getLog().error("All these resources are not added to the classloader:");
        for (Object r : resources) {
            if (r instanceof File) {
                 MavenLog.getLog().error("     " + ((File) r).getAbsolutePath());
            } else {
                MavenLog.getLog().error("     " + r.toString());
            }
        }
    }

    /**
     * Add a JAR file to the plugin classloader.
     * @param jarFile The JAR file
     * @return True if the file is added, false otherwise.
     */
    public boolean addJarResource(final File jarFile) {
        if (this.pluginContainer == null) {
            MavenLog.getLog().error("The MavenPluginClassloader is not initialized.");
            MavenLog.getLog().error("All these resources are not added to the classloader:");
            MavenLog.getLog().error("     " + jarFile.getAbsolutePath());
            return false;
        }
        try {
            this.pluginContainer.addJarResource(jarFile);
        } catch (PlexusContainerException ex) {
            return false;
        }
        return true;
    }

    /**
     * Add JAR files to the plugin classloader.
     * @param jarFiles List of JAR files. If null, nothing is added
     * @return True if all files are added, false otherwise.
     */
    public boolean addJarResources(final List<File> jarFiles) {
        boolean res = true;
        if (this.pluginContainer == null) {
            reportListError(jarFiles);
            res = false;
        } else if (jarFiles != null) {
            for (File file : jarFiles) {
                if (file.exists() && file.isFile()) {
                    try {
                        this.pluginContainer.addJarResource(file);
                    } catch (PlexusContainerException ex) {
                        MavenLog.getLog().error("Error while adding JAR to the plugin classloader : " + file.getName());
                        res = false;
                    }
                }
            }
        }
        return res;
    }

    /**
     * Add a resource to the plugin classloader.
     * @param urlResource The URL of the resource
     */
    public void addResource(final URL urlResource) {
        if (this.pluginRealm == null) {
            MavenLog.getLog().error("The MavenPluginClassloader is not initialized.");
            MavenLog.getLog().error("All these resources are not added to the classloader:");
            MavenLog.getLog().error("     " + urlResource.toString());
        } else {
            Method method;
            try {
                method = pluginRealm.getClass().getMethod("addConstituent", new Class[] {URL.class});
                method.invoke(this.pluginRealm, urlResource);
            } catch (SecurityException e) {
                throw new IllegalArgumentException("Unable to add the resource '" + urlResource + "'.", e);
            } catch (NoSuchMethodException e) {
                throw new IllegalArgumentException("Unable to add the resource '" + urlResource + "'.", e);
            } catch (IllegalArgumentException e) {
                throw new IllegalArgumentException("Unable to add the resource '" + urlResource + "'.", e);
            } catch (IllegalAccessException e) {
                throw new IllegalArgumentException("Unable to add the resource '" + urlResource + "'.", e);
            } catch (InvocationTargetException e) {
                throw new IllegalArgumentException("Unable to add the resource '" + urlResource + "'.", e);
            }
        }
    }

    /**
     * Add resources to the plugin classloader.
     * @param urlResources A list of resources URLs. If null, nothing is added
     */
    public void addResources(final List<URL> urlResources) {
        if (this.pluginRealm == null) {
            reportListError(urlResources);
        } else if (urlResources != null) {
            for (URL url : urlResources) {
                addResource(url);
            }
        }
    }

    /**
     * Get all URLs contained in the classloader.
     * @return List of URLs contained in the classloader, or null if error
     */
    public URL[] getResources() {
        if (this.pluginRealm == null) {
            MavenLog.getLog().error("The MavenPluginClassloader is not initialized.");
            return null;
        }


        Method method;
        try {
            method = pluginRealm.getClass().getMethod("getConstituents");
            return (URL[]) method.invoke(this.pluginRealm);
        } catch (SecurityException e) {
            throw new IllegalArgumentException("Unable to get the resources", e);
        } catch (NoSuchMethodException e) {
            throw new IllegalArgumentException("Unable to get the resources", e);
        } catch (IllegalArgumentException e) {
            throw new IllegalArgumentException("Unable to get the resources", e);
        } catch (IllegalAccessException e) {
            throw new IllegalArgumentException("Unable to get the resources", e);
        } catch (InvocationTargetException e) {
            throw new IllegalArgumentException("Unable to get the resources", e);
        }
    }
}
