/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MavenRepositoryFactory.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.maven;

import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.artifact.repository.ArtifactRepositoryFactory;
import org.apache.maven.artifact.repository.ArtifactRepositoryPolicy;
import org.apache.maven.artifact.repository.DefaultArtifactRepositoryFactory;
import org.apache.maven.artifact.repository.layout.ArtifactRepositoryLayout;
import org.apache.maven.artifact.repository.layout.DefaultRepositoryLayout;

/**
 * Class for simplify the creation of Maven repositories.
 * @author Vincent Michaud
 */
public final class MavenRepositoryFactory {

    /**
     * The artifact repository factory.
     */
    private static final ArtifactRepositoryFactory FACTORY = new DefaultArtifactRepositoryFactory();

    /**
     * The default layout.
     */
    public static final ArtifactRepositoryLayout DEFAULT_LAYOUT = new DefaultRepositoryLayout();

    /**
     * The legacy layout.
     */
    public static final ArtifactRepositoryLayout LEGACY_LAYOUT = new DefaultRepositoryLayout();


    /**
     * Update policy choice.
     */
    public static final String UPDATE_POLICY_NEVER = ArtifactRepositoryPolicy.UPDATE_POLICY_NEVER;

    /**
     * Update policy choice.
     */
    public static final String UPDATE_POLICY_ALWAYS = ArtifactRepositoryPolicy.UPDATE_POLICY_ALWAYS;

    /**
     * Update policy choice.
     */
    public static final String UPDATE_POLICY_DAILY = ArtifactRepositoryPolicy.UPDATE_POLICY_DAILY;

    /**
     * Update policy choice.
     */
    public static final String UPDATE_POLICY_INTERVAL = ArtifactRepositoryPolicy.UPDATE_POLICY_INTERVAL;


    /**
     * Checksum policy choice.
     */
    public static final String CHECKSUM_POLICY_FAIL = ArtifactRepositoryPolicy.CHECKSUM_POLICY_FAIL;

    /**
     * Checksum policy choice.
     */
    public static final String CHECKSUM_POLICY_WARN = ArtifactRepositoryPolicy.CHECKSUM_POLICY_WARN;

    /**
     * Checksum policy choice.
     */
    public static final String CHECKSUM_POLICY_IGNORE = ArtifactRepositoryPolicy.CHECKSUM_POLICY_IGNORE;

    
    /**
     * Constructor.
     */
    private MavenRepositoryFactory() {
    }

    /**
     * Create a new local repository.
     * @param id The ID of the repository
     * @param url The URL of the repository
     * @param uniqueVersion If each snapshot is considered as an unique version
     * @return The repository
     */
    public static ArtifactRepository createLocalRepository(final String id, final String url,
                                                           final boolean uniqueVersion) {
        return FACTORY.createDeploymentArtifactRepository(id, url, DEFAULT_LAYOUT, uniqueVersion);
    }

    /**
     * Create a new local repository.
     * @param id The ID of the repository
     * @param url The URL of the repository
     * @param uniqueVersion If each snapshot is considered as an unique version
     * @param layout The layout of the repository
     * @return The repository
     */
    public static ArtifactRepository createLocalRepository(final String id, final String url,
                                                           final boolean uniqueVersion,
                                                           final ArtifactRepositoryLayout layout) {
        return FACTORY.createDeploymentArtifactRepository(id, url, layout, uniqueVersion);
    }

    /**
     * Create a new remote repository.
     * @param id The ID of the repository
     * @param url The URL of the repository
     * @param releasesEnabled If the repository is enabled for releases
     * @param releasesUpdate The releases update policy : UPDATE_POLICY_NEVER, UPDATE_POLICY_DAILY,
     *                       UPDATE_POLICY_ALWAYS, UPDATE_POLICY_INTERVAL
     * @param releasesChecksum The releases checksum policy : CHECKSUM_POLICY_FAIL,
     *                         CHECKSUM_POLICY_WARN, CHECKSUM_POLICY_IGNORE
     * @param snapshotsEnabled If the repository is enabled for snapshots
     * @param snapshotsUpdate The releases update policy : UPDATE_POLICY_NEVER, UPDATE_POLICY_DAILY,
     *                                                     UPDATE_POLICY_ALWAYS, UPDATE_POLICY_INTERVAL
     * @param snapshotsChecksum The snapshots checksum policy : CHECKSUM_POLICY_FAIL,
     *                          CHECKSUM_POLICY_WARN, CHECKSUM_POLICY_IGNORE
     * @param layout The layout of the repository
     * @return The repository
     */
    public static ArtifactRepository createRemoteRepository(
            final String id, final String url,
            final boolean releasesEnabled, final String releasesUpdate, final String releasesChecksum,
            final boolean snapshotsEnabled, final String snapshotsUpdate, final String snapshotsChecksum,
            final ArtifactRepositoryLayout layout) {

        ArtifactRepositoryPolicy releasesPolicy = new ArtifactRepositoryPolicy(releasesEnabled, releasesUpdate,
                                                                               releasesChecksum);
        ArtifactRepositoryPolicy snapshotsPolicy = new ArtifactRepositoryPolicy(snapshotsEnabled, snapshotsUpdate,
                                                                                snapshotsChecksum);
        return FACTORY.createArtifactRepository(id, url, layout, snapshotsPolicy, releasesPolicy);
    }

    /**
     * Create a new releases repository.
     * @param id The ID of the repository
     * @param url The URL of the repository
     * @param releasesUpdate The releases update policy : UPDATE_POLICY_NEVER, UPDATE_POLICY_DAILY,
     *                       UPDATE_POLICY_ALWAYS, UPDATE_POLICY_INTERVAL
     * @param releasesChecksum The releases checksum policy : CHECKSUM_POLICY_FAIL,
     *                         CHECKSUM_POLICY_WARN, CHECKSUM_POLICY_IGNORE
     * @return The repository
     */
    public static ArtifactRepository createReleasesRemoteRepository(final String id, final String url,
                                                                    final String releasesUpdate,
                                                                    final String releasesChecksum) {
        ArtifactRepositoryPolicy releasesPolicy = new ArtifactRepositoryPolicy(true, releasesUpdate, releasesChecksum);
        return FACTORY.createArtifactRepository(id, url, DEFAULT_LAYOUT, defaultPolicy(false), releasesPolicy);
    }

    /**
     * Create a new repository.
     * @param id The ID of the repository
     * @param url The URL of the repository
     * @param snapshotsUpdate The releases update policy : UPDATE_POLICY_NEVER, UPDATE_POLICY_DAILY,
     *                                                     UPDATE_POLICY_ALWAYS, UPDATE_POLICY_INTERVAL
     * @param snapshotsChecksum The snapshots checksum policy : CHECKSUM_POLICY_FAIL,
     *                          CHECKSUM_POLICY_WARN, CHECKSUM_POLICY_IGNORE
     * @return The repository
     */
    public static ArtifactRepository createSnapshotsRemoteRepository(final String id, final String url,
                                                                     final String snapshotsUpdate,
                                                                     final String snapshotsChecksum) {
        ArtifactRepositoryPolicy snapshotsPolicy = new ArtifactRepositoryPolicy(true, snapshotsUpdate, snapshotsChecksum);
        return FACTORY.createArtifactRepository(id, url, DEFAULT_LAYOUT, snapshotsPolicy, defaultPolicy(false));
    }

    /**
     * Create a default policy.
     * @param enabled Whether the repository is enabled
     * @return The policy
     */
    private static ArtifactRepositoryPolicy defaultPolicy(final boolean enabled) {
        return new ArtifactRepositoryPolicy(enabled, UPDATE_POLICY_DAILY, CHECKSUM_POLICY_WARN);
    }

}
