/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConsoleScanner.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Thread to detect ENTER key user action.
 * @author Alexandre Deneux
 * @author Bilel Souadnia
 */
public class ConsoleScanner extends Thread {

    /**
     * Action executed when ENTER is pushed.
     */
    public interface Action {

        /**
         * Function launched by the console scanner.
         */
        void execute();
    }

    /**
     * The action to do when user push enter.
     */
    private Action action;

    /**
     * The buffered reader.
     */
    private BufferedReader buffer;

    /**
     * The message for stop thread.
     */
    private String message = null;

    /**
     * If the console scanner is stopped.
     */
    private boolean stopped = false;

    /**
     * If the buffer is cleared after execution of action.
     */
    private boolean clearBuffer = true;

    /**
     * Constructor.
     * @param action The executed action
     * @param buffer The buffered reader
     */
    public ConsoleScanner(final Action action, final BufferedReader buffer) {
        setDaemon(true);
        this.action = action;
        this.buffer = buffer;
    }

    /**
     * Constructor.
     * @param action The executed action
     */
    public ConsoleScanner(final Action action) {
        setDaemon(true);
        this.action = action;
        this.buffer = new BufferedReader(new InputStreamReader(System.in));
    }

    /**
     * Set the exit message.
     * @param msg The message
     */
    public void setExitMessage(final String msg) {
        this.message = msg;
    }

    /**
     * Get the exit message.
     * @return The message
     */
    public String getExitMessage() {
        return this.message;
    }

    /**
     * Set if the buffer is cleared after execution of action.
     * @param cb If true, the buffer is cleared
     */
    public void setClearBuffer(final boolean cb) {
        this.clearBuffer = cb;
    }

    /**
     * Whether or not the buffer is cleared after execution of action.
     * @return Boolean
     */
    public boolean getClearBuffer() {
        return this.clearBuffer;
    }

    /**
     * Whether or not the thread is stopped.
     * @return Boolean
     */
    public boolean isStopped() {
        return this.stopped;
    }

    /**
     * Content of the thread.
     */
    @Override
    public void run() {
        try {
            String read;

            while (true) {
                read = this.buffer.readLine();

                if (read != null) {
                    if (this.message != null && read.compareTo(this.message) == 0) {
                        this.stopped = true; return;
                    } else {
                        this.action.execute();
                        if (this.clearBuffer) {
                            while (this.buffer.ready()) {
                                this.buffer.read();
                            }
                        }
                    }
                }
            }
        } catch (IOException ex) {
            Logger.getLogger(ConsoleScanner.class.getName()).log(Level.SEVERE,
                    "Error from console scanner. Service desactived.", ex);
        }
    }
}
