/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: RepeatedTask.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.utils;

import java.util.Timer;
import java.util.TimerTask;


/**
 * Repeat with a defined time interval a task.
 * @author Vincent Michaud
 */
public abstract class RepeatedTask {


    /**
     * Interface for determine the action before
     * and after the execution of task.
     */
    public interface EventAction {

        /**
         * Task executed.
         */
        void process();
    }

    /****************************************************/
    /*                        Fields                    */
    /****************************************************/

    /**
     * Default task interval.
     */
    private static final long DEFAULT_TASK_INTERVAL = 1000;

    /**
     * Interval between two tasks.
     */
    private long taskInterval = DEFAULT_TASK_INTERVAL;

    /**
     * If the timer is started.
     */
    private boolean started = false;    

    /**
     * The timer.
     */
    private Timer timer = null;

    /**
     * The timer task.
     */
    private TimerTask task = null;    
    
    /**
     * The action executed before the task.
     */
    private EventAction beforeAction = null;

    /**
     * The action executed after the task.
     */
    private EventAction afterAction = null;
    

    /****************************************************/
    /*                     Functions                    */
    /****************************************************/


    /**
     * Create a new repeated task.
     */
    public RepeatedTask() {
    }

    /**
     * The repeated task to execute.
     */
    protected abstract void executeTask();

    /**
     * Whether or not the task is started.
     * @return True if he task is started
     */
    public synchronized boolean isStarted() {
        return this.started;
    }

    /**
     * Force the execution of the task.
     */
    public synchronized void launchNow() {
        if (this.beforeAction != null) {
            this.beforeAction.process();
        }
        executeTask();
        if (this.afterAction != null) {
            this.afterAction.process();
        }
    }

    /**
     * Get the value of task interval in millisecond.
     * @return The value of interval.
     */
    public long getInterval() {
        return this.taskInterval;
    }
    
    /**
     * Set the value of task interval in millisecond.
     * @param interval New value of interval
     */
    public void setInterval(final long interval) {
        this.taskInterval = interval;
        scheduleTimer(0);
    }

    /**
     * Set the action executed before the task.
     * @param action The action, or null for removing action.
     */
    public void setActionBeforeScan(final EventAction action) {
        this.beforeAction = action;
    }

    /**
     * Set the action executed after the task.
     * @param action The action, or null for removing action.
     */
    public void setActionAfterScan(final EventAction action) {
        this.afterAction = action;
    }

    /**
     * Get the action executed before and after the task.
     * @return The action
     */
    public EventAction getActionBeforeScan() {
        return this.beforeAction;
    }

    /**
     * Get the action executed before and after the task.
     * @return The action
     */
    public EventAction getActionAfterScan() {
        return this.afterAction;
    }

    /**
     * Create a new task.
     * @return New task
     */
    private TimerTask newTimerTask() {
        return new TimerTask() {
            public void run() {
                launchNow();
            }
        };
    }

    /**
     * Reset the timer.
     */
    private void resetTimer() {
        if (this.timer != null) {
            this.timer.cancel();
            this.timer = null;
        }
        if (this.task != null) {
            this.task.cancel();
            this.task = null;
        }
    }

    /**
     * Schedule the timer.
     * @param startIn Delay before the begin of task
     */
    private void scheduleTimer(final long startIn) {
        if (this.started) {
            resetTimer();
            if (this.taskInterval > 0) {
                this.timer = new Timer(true);
                this.task = newTimerTask();
                this.timer.schedule(this.task, startIn, this.taskInterval);
            }
        }
    }

    /**
     * Start the repeated task.
     */
    public void start() {
        start(0L);
    }

    /**
     * Start the repeated task.
     * @param startIn Delay before the begin of task
     */
    public synchronized void start(final long startIn) {
        if (!this.started) {
            this.started = true;
            scheduleTimer(startIn);
        }
    }

    /**
     * Stop the repeated task.
     */
    public synchronized void stop() {
        if (this.started) {
            resetTimer();
            this.started = false;
        }
    }
}
