/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Scanner.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.utils;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Scanner witch detect all modifications of registered files.
 * @author Vincent Michaud
 */
public class Scanner extends RepeatedTask {

    /****************************************************/
    /*                      Interfaces                  */
    /****************************************************/

    /**
     * Define specific action on a file.
     */
    public interface Listener {

        /**
         * Perform action on the modified file.
         * @param canonicalPath Added file
         * @param typeAction The type of modification : Scanner.ADDED_ACTION,
         *                   Scanner.CHANGED_ACTION or Scanner.DELETED_ACTION
         */
        void reportAction(final String canonicalPath, final int typeAction);
    }

    /**
     * Processing different mode for listeners inheritance.
     */
    private interface ListenerCaller {

        /**
         * Call all listeners of the element described.
         * @param info The scan information of the element
         * @param typeAction The type of modification : Scanner.ADDED_ACTION,
         *                   Scanner.CHANGED_ACTION or Scanner.DELETED_ACTION
         */
        void callListeners(final ScanInfo info, final int typeAction);
    }

    /**
     * Processing different mode for managing filter.
     */
    private interface FilterMethod {

        /**
         * Call all listeners of the element described.
         * @param config The configuration of current file
         * @param file The file to filter
         * @return True if the file is accepted, false otherwise
         */
        boolean accept(final DirConfig config, final File file);
    }


    /****************************************************/
    /*                      Constants                   */
    /****************************************************/


    /**
     * A type of action on scanned files.
     */
    public static final int ADDED_ACTION = 0;

    /**
     * A type of action on scanned files.
     */
    public static final int CHANGED_ACTION = 1;

    /**
     * A type of action on scanned files.
     */
    public static final int DELETED_ACTION = 2;

    /**
     * A type of behaviour for the listeners inheritance.
     */
    public static final int NORMAL_INHERITANCE_MODE = 0;

    /**
     * A type of behaviour for the listeners inheritance.
     */
    public static final int COMPLETE_INHERITANCE_MODE = 1;

    /**
     * A type of behaviour for the listeners inheritance.
     */
    public static final int EXCLUSIVE_INHERITANCE_MODE = 2;

    /**
     * Default initial size of the StringBuffer.
     */
    private static final int BUFFER_SIZE = 256;


    /****************************************************/
    /*                        Fields                    */
    /****************************************************/

    /**
     * Whether or not an initial scan will report all files as being added.
     */
    private boolean reportExistingFiles = true;

    /**
     * Determine if scanned elements have been modified since the last scan.
     */
    private int[] nbElmtsModified = {0, 0, 0};

    /**
     * Date of the last scan.
     */
    private long lastScan = -1;

    /**
     * The processing function of the inheritance mode of listeners.
     */
    private ListenerCaller inheritanceManager = null;

    /**
     * The method used to filter files.
     */
    private FilterMethod filterMethod = null;

    /**
     * List of elements to scan.
     */
    private List<ScannedElement> scanElements = new LinkedList<ScannedElement>();

    /**
     * List of configured directories.
     */
    private Map<String, DirConfig> directoriesConfig = new HashMap<String, DirConfig>();

    /**
     * List of file listeners.
     */
    private Map<String, List<Listener>> fileListeners = new HashMap<String, List<Listener>>();

    /**
     * Excluded elements.
     */
    private Set<String> excludedElements = new HashSet<String>();

    /**
     * Last scan done.
     */
    private List<ScanInfo> oldScan = new ArrayList<ScanInfo>();
    
    /**
     * Current scan.
     */
    private List<ScanInfo> newScan = new ArrayList<ScanInfo>();

    /**
     * Whether or not it is the first scan.
     */
    private boolean firstScan = true;

    /**
     * Path of directories, used in recursives functions.
     */
    private StringBuffer recPath = null;

    /**
     * Temporary file.
     */
    private File tmpFile = null;

    /**
     * Temporary directory.
     */
    private String tmpPath = null;

    /**
     * Temporary configuration.
     */
    private DirConfig tmpConfig = null;


    /****************************************************/
    /*                     Functions                    */
    /****************************************************/


    /**
     * Create a new scanner.
     */
    public Scanner() {
        this.recPath = new StringBuffer(BUFFER_SIZE);
        this.inheritanceManager = this.normalCaller;
        this.filterMethod = this.simpleFilter;
    }

    /**
     * Add a file to scan.
     * @param path Path of the file
     * @param listener Called actions when scanner detect event
     */
    public synchronized void addScanFile(final String path, final Listener listener) {
        if (listener == null) {
            throw new NullPointerException("Listener is necesary for the file : " + path);
        }
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            this.scanElements.add(new ScannedElement(canoPath, false));
            List<Listener> listeners = new ArrayList<Listener>();
            listeners.add(listener);
            this.fileListeners.put(canoPath, listeners);
        }
    }

    /**
     * Add a directory to scan.
     * @param path Path of the directory
     * @param listener Called actions when scanner detect event
     */
    public synchronized void addScanDirectory(final String path, final Listener listener) {
        addScanDirectory(path, listener, true, false, null);
    }

    /**
     * Add a directory to scan.
     * @param path Path of the directory
     * @param listener Called actions when scanner detect event
     * @param recursive Whether or not the scan is recursive
     * @param reportOnDirectory Whether or not events are reported on the directory
     * @param filter File filter
     */
    public synchronized void addScanDirectory(final String path, final Listener listener, final boolean recursive,
                                                 final boolean reportOnDirectory, final FileFilter filter) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            /* // Thow an exception if a conflict is detected between two scanned directory of the scanner
             Iterator<ScannedElement> it = this.scanElements.iterator();
            ScannedElement elmt = null;
            while (it.hasNext()) {
                elmt = it.next();
                if (elmt.isDir && (elmt.canoPath.startsWith(canoPath) || canoPath.startsWith(elmt.canoPath))) {
                    throw new RuntimeException("There is a conflict with scanned directories:\n    "
                                             + canoPath + "\n    " + elmt.canoPath);
                }
            } */
            this.scanElements.add(new ScannedElement(canoPath, true));
            setDirectoryConfig(canoPath, recursive, reportOnDirectory, filter, listener);
        }
    }

    /**
     * Set the configuration of a directory. Listeners previously added on the element are kept
     * @param path Path of the directory
     * @param recursive Whether or not the scan is recursive
     * @param reportOnDirectory Whether or not events are reported on the directory
     * @param filter File filter
     */
    public synchronized void setDirectoryConfig(final String path, final boolean recursive,
                                                final boolean reportOnDirectory, final FileFilter filter) {
        setDirectoryConfiguration(path, recursive, reportOnDirectory, filter, null);
    }

    /**
     * Set the configuration of a directory. All listeners added previously on the element are removed
     * @param path Path of the directory
     * @param recursive Whether or not the scan is recursive
     * @param reportOnDirectory Whether or not events are reported on the directory
     * @param filter File filter
     * @param listener The new listener of the element
     */
    public synchronized void setDirectoryConfig(final String path, final boolean recursive,
                                                final boolean reportOnDirectory, final FileFilter filter,
                                                final Listener listener) {
        if (listener == null) {
            throw new NullPointerException("Listener is necesary for the file : " + path);
        }
        setDirectoryConfiguration(path, recursive, reportOnDirectory, filter, listener);
    }

    /**
     * Set the configuration of a directory.
     * @param path Canonical path of the directory
     * @param recursive Whether or not the scan is recursive
     * @param reportOnDirectory Whether or not events are reported on the directory
     * @param filter File filter
     * @param listener Listener of the directory, or null for keep listeners already added
     */
    private void setDirectoryConfiguration(final String path, final boolean recursive,
                                           final boolean reportOnDirectory, final FileFilter filter,
                                           final Listener listener) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            List<Listener> listeners = null;
            if (listener != null) {
                listeners = new ArrayList<Listener>();
                listeners.add(listener);
            }

            DirConfig config = this.directoriesConfig.get(canoPath);
            if (config == null) {
                config = new DirConfig(recursive, reportOnDirectory, filter, listeners, canoPath);
                putDirectoryConfig(canoPath, config);
            } else {
                config.recursive = recursive;
                config.reportOnDirectory = reportOnDirectory;
                config.filter = filter;
                config.passive = false;
                if (listeners != null) {
                    config.listeners = listeners;
                }
            }
        }
    }

    /**
     * Update the properties of DirConfig object when an directory
     * is added to the scanner.
     * @param canoPathA The canonical path of the element configured
     * @param configA The configuration of the directory
     */
    private void putDirectoryConfig(final String canoPathA, final DirConfig configA) {
        int lengthA = canoPathA.length(), lengthB;
        int lengthParentA = -1;
        String canoPathB;

        if (configA.parentConfig != null) {
            lengthParentA = configA.parentConfig.canoPath.length();
        }

        for (DirConfig configB : this.directoriesConfig.values()) {
            canoPathB = configB.canoPath;
            lengthB = canoPathB.length();

            // Check if one element is parent of the other.
            if (lengthB < lengthA) {
                if (canoPathA.startsWith(canoPathB)) {
                    if (lengthB > lengthParentA) {
                        changeParent(configA, configB);
                    }
                }
            } else if (canoPathB.startsWith(canoPathA) && lengthB != lengthA) {
                if (configB.parentConfig == null || lengthA > configB.parentConfig.canoPath.length()) {
                    changeParent(configB, configA);
                }
            }
        }

        if (this.inheritanceManager != this.completeCaller) {
            updateNormalInheritance(configA);
        }
        directoriesConfig.put(canoPathA, configA);
    }

    /**
     * Link a configuration to its parent configuration.
     * If configuration is passive (only listeners are effective),
     * the configuration is copied from the first none passive parent.
     * @param config The configuration to link
     * @param parent The new parent of the configuration
     */
    private void changeParent(final DirConfig config, final DirConfig parent) {
        config.parentConfig = parent;
        if (config.passive) {
            updatePassiveConfiguration(config);
        }
    }

    /**
     * Find the first parent configuration which is not passive and
     * copy its fields into the passive configuration. So, the passive
     * configuration look "transparent".
     * @param config A passive configuration
     */
    private void updatePassiveConfiguration(final DirConfig config) {
        DirConfig lastConfig = null, crtConfig;
        crtConfig = config.parentConfig;
        while (crtConfig != null && crtConfig.passive) {
            lastConfig = crtConfig;
            crtConfig = crtConfig.parentConfig;
        }
        if (crtConfig == null) {
            crtConfig = lastConfig;
        }
        if (crtConfig != null) {
            config.filter = crtConfig.filter;
            config.recursive = crtConfig.recursive;
            config.reportOnDirectory = crtConfig.reportOnDirectory;
        }
    }

    /**
     * Find the first parent configuration which has listeners and
     * copy them into current configuration. So, any configuration
     * has listeners inherited from its first parent.
     * @param config The configuration to update
     */
    private void updateNormalInheritance(final DirConfig config) {
        if (config.listeners == null) {
            DirConfig crtConfig = config.parentConfig;
            while (crtConfig != null) {
                if (crtConfig.listeners != null) {
                    config.listeners = crtConfig.listeners;
                    break;
                }
                crtConfig = crtConfig.parentConfig;
            }
        }
    }

    /**
     * Find the first parent configuration which has not listeners and
     * check if listeners are not duplicated between a configuration and its parent.
     * @param config The configuration to update
     */
    private void updateCompleteInheritance(final DirConfig config) {
        if (config.listeners != null) {
            DirConfig crtConfig = config.parentConfig;
            while (crtConfig != null) {
                if (crtConfig.listeners == config.listeners) {
                    config.listeners = null;
                    break;
                }
                crtConfig = crtConfig.parentConfig;
            }
        }
    }

    /**
     * Remove the configuration of an element (filters, recursive mode and reportOnDirectory).
     * @param path The path of the element
     * @param removeListeners Determine if listeners previously added are kept or not.
     */
    public synchronized void removeDirectoryConfig(final String path, final boolean removeListeners) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            removeDirConfig(canoPath, removeListeners);
        }
    }

    /**
     * Remove the configuration of an element (filters, recursive mode and reportOnDirectory).
     * @param canoPath The canonical path of the element
     * @param removeListeners Determine if listeners previously added are kept or not.
     */
    private void removeDirConfig(final String canoPath, final boolean removeListeners) {
        if (removeListeners) {
            DirConfig removedConfig = this.directoriesConfig.remove(canoPath);
            if (removedConfig != null) {
                for (DirConfig config : this.directoriesConfig.values()) {
                    if (config.parentConfig == removedConfig) {
                        config.parentConfig = removedConfig.parentConfig;
                    }
                }
            }
        } else {
            DirConfig config = this.directoriesConfig.get(canoPath);
            if (config != null) {
                config.passive = true;
                updatePassiveConfiguration(config);
            }
        }
    }

    /**
     * Remove a scanned element, its listeners, and its configuration,
     * if the element is found only.
     * @param path The path of the element
     *//*
    public synchronized void removeScanElement(final String path) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            boolean removed = false;
            Iterator<ScannedElement> it = this.scanElements.iterator();
            ScannedElement elmt = null;
            while (it.hasNext()) {
                elmt = it.next();
                if (elmt.canoPath.equals(canoPath)) {
                    it.remove();
                    removed = true;
                    break;
                }
            }

            if (removed) {
                this.fileListeners.remove(canoPath);
                removeDirConfig(canoPath, true);
                
                Iterator<ScanInfo> it2 = this.oldScan.iterator();
                ScanInfo info = null;
                while (it2.hasNext()) {
                    info = it2.next();
                    if (info.canoPath.startsWith(canoPath) && (!elmt.isDir || info.config != null)) {
                        it2.remove();
                    }
                }
            }
        }
    }*/

    /**
     * Determine if an Element is in the scanElements List.
     * @param path The path of the element to find
     * @return true if the path is in the Map, false else
     */
    public synchronized boolean containsElement(final String path) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            for (ScannedElement elmt : this.scanElements) {
                if (elmt.canoPath.equals(canoPath)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Add a listener to a file.
     * If the path represents a directory, this listener will be only
     * effective on the directory and not on its children (Inheritance
     * of the listener is not possible). Listener inheritance is possible
     * with the function addDirectoryListener().
     * @param path Path of the file
     * @param listener Called action when scanner detect event
     */
    public synchronized void addFileListener(final String path, final Listener listener) {
        if (listener == null) {
            throw new NullPointerException("Listener is necesary for the file : " + path);
        }
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            List<Listener> listeners = this.fileListeners.get(canoPath);
            if (listeners == null) {
                listeners = new ArrayList<Listener>();
                this.fileListeners.put(canoPath, listeners);
            }
            listeners.add(listener);
        }
    }

    /**
     * Add a listener to a directory. The listener can be inherited to
     * children of the directory.
     * @param path Path of the directory
     * @param listener Called action when scanner detect event
     */
    public synchronized void addDirectoryListener(final String path, final Listener listener) {
        if (listener == null) {
            throw new NullPointerException("Listener is necesary for the file : " + path);
        }
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            DirConfig config = this.directoriesConfig.get(canoPath);
            if (config == null) {
                List<Listener> listeners = new ArrayList<Listener>();
                listeners.add(listener);
                config = new DirConfig(listeners, canoPath);
                putDirectoryConfig(canoPath, config);
            } else {
                if (config.listeners == null) {
                    config.listeners = new ArrayList<Listener>();
                }
                config.listeners.add(listener);
            }
            
        }
    }

    /**
     * Determine if a listener is affected to an element.
     * @param path The path of the element to find
     * @param listener The listener of the element to find
     * @return true if the path is in the Map, false else
     */
    public synchronized boolean containsListener(final String path, final Listener listener) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            List<Listener> listeners = this.fileListeners.get(canoPath);
            if (listeners != null && listeners.contains(listener)) {
                return true;
            }
            DirConfig config = this.directoriesConfig.get(canoPath);
            return (config != null && config.listeners != null && config.listeners.contains(listener));
        }
        return false;
    }

   /**
     * Remove a listener to an element.
     * @param path Path of the file
     * @param listener Listener to remove
     */
    public synchronized void removeListener(final String path, final Listener listener) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            boolean res;
            ScannedElement elmt = findElement(canoPath);
            List<Listener> listeners;
            DirConfig config = this.directoriesConfig.get(canoPath);

            if (config != null) {
                listeners = config.listeners;
                if (listeners != null) {
                    do {
                        res = listeners.remove(listener);
                    } while (res);
                    if (listeners.isEmpty()) {
                        config.listeners = null;
                        if (config.passive) {
                            this.directoriesConfig.remove(canoPath);
                        } else if (this.inheritanceManager != this.completeCaller) {
                            updateNormalInheritance(config);
                        }
                    }
                }
            }
            listeners = this.fileListeners.get(canoPath);
            if (listeners != null) {
                do {
                    res = listeners.remove(listener);
                } while (res);
                if (listeners.isEmpty()) {
                    this.fileListeners.remove(canoPath);
                }
            }
        }
    }

    /**
     * Get listeners of a specific element.
     * @param path The path of the element
     * @return Listeners or null, if not any listener is found
     */
    public synchronized List<Listener> getListeners(final String path) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            List<Listener> result = new LinkedList<Listener>();
            List<Listener> listeners = this.fileListeners.get(canoPath);
            if (listeners != null) {
                result.addAll(listeners);
            }
            DirConfig config = this.directoriesConfig.get(canoPath);
            if (config != null && config.listeners != null) {
                result.addAll(config.listeners);
            }
            if (result.size() > 0) {
                return result;
            }
        }
        return null;
    }

    /**
     * Find an element.
     * @param canoPath The canonical path of the element
     * @return The element
     */
    private ScannedElement findElement(final String canoPath) {
        for (ScannedElement elmt : this.scanElements) {
            if (elmt.canoPath.equals(canoPath)) {
                return elmt;
            }
        }
        return null;
    }

    /**
     * Determine if scanned elements have changed since last scan.
     * @return True, if scanned elements have changed
     */
    public synchronized boolean hasChanged() {
        return (this.nbElmtsModified[ADDED_ACTION]   != 0
             || this.nbElmtsModified[CHANGED_ACTION] != 0
             || this.nbElmtsModified[DELETED_ACTION] != 0);
    }

    /**
     * Determine if elements have been added since last scan.
     * @return Number of added files
     */
    public synchronized int nbAdded() {
        return this.nbElmtsModified[ADDED_ACTION];
    }

    /**
     * Determine if elements have been changed since last scan.
     * @return Number of modified files
     */
    public synchronized int nbChanged() {
        return this.nbElmtsModified[CHANGED_ACTION];
    }

    /**
     * Determine if elements have been deleted since last scan.
     * @return Number of deleted files
     */
    public synchronized int nbDeleted() {
        return this.nbElmtsModified[DELETED_ACTION];
    }

    /**
     * Get the date of last scan.
     * @return The date of last scan, or -1 if error.
     */
    public synchronized long lastScan() {
        return this.lastScan;
    }

    /**
     * Reset the last scan done.
     */
    public synchronized void resetScan() {
        this.nbElmtsModified[ADDED_ACTION]   = 0;
        this.nbElmtsModified[CHANGED_ACTION] = 0;
        this.nbElmtsModified[DELETED_ACTION] = 0;
        this.oldScan.clear();
        this.newScan.clear();
        this.firstScan = true;
        this.lastScan = -1;
    }

    /**
     * Exclude element of scanning.
     * @param path Path of the element
     */
    public synchronized void exclusion(final String path) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            this.excludedElements.add(canoPath);
        }
    }

    /**
     * Cancel exclusion of an element.
     * @param path Path of the element
     */
    public synchronized void cancelExclusion(final String path) {
        String canoPath = getCanonicalPath(path);
        if (canoPath != null) {
            this.excludedElements.remove(canoPath);
        }
    }

  /**
   * Whether or not an initial scan will report all files as being added.
   * @param reportExistingFiles if true, all files found on initial scan
   * will be reported as being added, otherwise not
   */
    public synchronized void setReportFilesOnStartup(final boolean reportExistingFiles) {
        this.reportExistingFiles = reportExistingFiles;
    }

    /**
     * Determine if filter are inherited from parents.
     * If filters are inherited from parents, a file is validated only
     * if all filters accept the file. In the other case, a file is validated
     * if the filter of the file accepts its. All parents filters are ignored.
     * @param inherited True for inherit filters from parents, false otherwise
     */
    public synchronized void setFilterInheritance(final boolean inherited) {
        if (inherited) {
            this.filterMethod = inheritedFilter;
        } else {
            this.filterMethod = simpleFilter;
        }
    }

    /**
     * This method not take care of parent filter.
     * Only the current filter is effective.
     */
    private final FilterMethod simpleFilter = new FilterMethod() {
        public boolean accept(final DirConfig config, final File file) {
            return (config.filter == null || config.filter.accept(file));
        }
    };

    /**
     * This method not take care of parent filter.
     * Only the current filter is effective.
     */
    private final FilterMethod inheritedFilter = new FilterMethod() {
        public boolean accept(final DirConfig c, final File file) {
            DirConfig config = c;
            do {
                if (config.filter != null && !config.filter.accept(file)) {
                    return false;
                } 
                config = config.parentConfig;
            } while (config != null);
            return true;
        }
    };

    /**
     * Determine the behaviour of the scanner for the inheritance of listeners.
     * @param listenersInheritanceMode The mode choosed. Values allowed are:
     *        Scanner.NORMAL_INHERITANCE_MODE, Scanner.EXCLUSIVE_INHERITANCE_MODE and
     *        Scanner.COMPLETE_INHERITANCE_MODE.<BR /><BR />
     *        In every instance, if an element has at least one listener, this last is called.<BR />
     *        Specificities of modes are the following : <BR />
     *        NORMAL_INHERITANCE_MODE: Listeners from the first parent element are called<BR />
     *        COMPLETE_INHERITANCE_MODE: Listeners from all parent elements are called<BR />
     *        EXCLUSIVE_INHERITANCE_MODE: If an element has specific listeners, they are called.
     *        In this case, parent listeners are not called. If the element has not specific listeners,
     *        the behaviour is the same that NORMAL_INHERITANCE_MODE.
     */
    public synchronized void setListenersInheritanceMode(final int listenersInheritanceMode) {
        switch (listenersInheritanceMode) {
            case COMPLETE_INHERITANCE_MODE:
                if (this.inheritanceManager != this.completeCaller) {
                    this.inheritanceManager  = this.completeCaller;
                    for (DirConfig config : this.directoriesConfig.values()) {
                        updateCompleteInheritance(config);
                    }
                }
                break;
            case EXCLUSIVE_INHERITANCE_MODE:
                if (this.inheritanceManager != this.exclusiveCaller) {
                    this.inheritanceManager  = this.exclusiveCaller;
                    for (DirConfig config : this.directoriesConfig.values()) {
                        updateNormalInheritance(config);
                    }
                }
                break;
            default:
                if (this.inheritanceManager != this.normalCaller) {
                    this.inheritanceManager  = this.normalCaller;
                    for (DirConfig config : this.directoriesConfig.values()) {
                        updateNormalInheritance(config);
                    }
                }
                break;
        }
    }

    /**
     * Call listeners of a file. If an element has at least one listener, this
     * last is called. Listeners from the first parent element are also called.
     */
    private final ListenerCaller normalCaller = new ListenerCaller() {
        public void callListeners(final ScanInfo info, final int typeAction) {
            nbElmtsModified[typeAction]++;
            List<Listener> listeners = fileListeners.get(info.canoPath);
            if (listeners != null) {
                for (Listener l : listeners) {
                    l.reportAction(info.canoPath, typeAction);
                }
            }

            if (info.config != null) {
                for (Listener l : info.config.listeners) {
                    l.reportAction(info.canoPath, typeAction);
                }
            }
        }
    };

    /**
     * Call listeners of a file. If an element has at least one listener, this
     * last is called and parent listeners are not called. Otherwise, listeners
     * from the first parent element are called.
     */
    private final ListenerCaller exclusiveCaller = new ListenerCaller() {
        public void callListeners(final ScanInfo info, final int typeAction) {
            nbElmtsModified[typeAction]++;
            List<Listener> listeners = fileListeners.get(info.canoPath);
            if (listeners != null) {
                for (Listener l : listeners) {
                    l.reportAction(info.canoPath, typeAction);
                }
            } else if (info.config != null) {
                for (Listener l : info.config.listeners) {
                    l.reportAction(info.canoPath, typeAction);
                }
            }
        }
    };

    /**
     * Call listeners of a file. If an element has at least one listener, this
     * last is called. Listeners from all parent elements are also called.
     */
    private final ListenerCaller completeCaller = new ListenerCaller() {
        public void callListeners(final ScanInfo info, final int typeAction) {
            nbElmtsModified[typeAction]++;
            List<Listener> listeners = fileListeners.get(info.canoPath);
            if (listeners != null) {
                for (Listener l : listeners) {
                    l.reportAction(info.canoPath, typeAction);
                }
            }

            DirConfig config = info.config;
            while (config != null) {
                if (config.listeners != null) {
                    for (Listener l : config.listeners) {
                        l.reportAction(info.canoPath, typeAction);
                    }
                }
                config = config.parentConfig;
            }
        }
    };

    /**
     * Scan a directory.
     * Warning, before calling this function : <br />
     * <code>recPath</code> and <code>tmpPath</code> must contains the canonical
     * path of <code>dir</code>, so, not ending with '/' or '\'.
     * @param dir The current directory
     * @param parentConfig Configuration of the directory
     */
    private void scanDirectory(final File dir, final DirConfig parentConfig) {

        if (parentConfig.reportOnDirectory) {
            if (this.filterMethod.accept(parentConfig, dir)) {
                this.newScan.add(new ScanInfo(this.tmpPath, dir.lastModified(), parentConfig));
            } else {
                return;
            }
        }

        String[] childNames = dir.list();

        if (childNames != null) {
            this.recPath.append(File.separatorChar);
            int ind = this.recPath.length();

            if (parentConfig.recursive) {
                for (String childName : childNames) {
                    this.tmpFile = new File(dir, childName);

                    if (this.tmpFile.isDirectory()) {
                        this.recPath.append(childName);
                        this.tmpPath = this.recPath.toString();
                        if (!this.excludedElements.contains(this.tmpPath)) {
                            this.tmpConfig = this.directoriesConfig.get(this.tmpPath);
                            if (this.tmpConfig == null) {
                                scanDirectory(this.tmpFile, parentConfig);
                            } else {
                                scanDirectory(this.tmpFile, this.tmpConfig);
                            }
                        }
                        this.recPath.setLength(ind);

                    } else if (this.filterMethod.accept(parentConfig, this.tmpFile)) {
                        this.recPath.append(childName);
                        this.tmpPath = this.recPath.toString();
                        if (!this.excludedElements.contains(this.tmpPath)) {
                            this.newScan.add(new ScanInfo(this.tmpPath, this.tmpFile.lastModified(), parentConfig));
                        }
                        this.recPath.setLength(ind);
                    }
                }
            } else {
                for (String childName : childNames) {
                    this.tmpFile = new File(dir, childName);
                    if (!this.tmpFile.isDirectory() && this.filterMethod.accept(parentConfig, this.tmpFile)) {
                        this.recPath.append(childName);
                        this.tmpPath = this.recPath.toString();
                        if (!this.excludedElements.contains(this.tmpPath)) {
                            this.newScan.add(new ScanInfo(this.tmpPath, this.tmpFile.lastModified(), parentConfig));
                        }
                        this.recPath.setLength(ind);
                    }
                }
            }
        }
    }

    /**
     * Scan all elements for modifications.
     */
    private void scanElements() {
        DirConfig config = null;
        this.newScan = new ArrayList<ScanInfo>();

        for (ScannedElement elmt : this.scanElements) {
            if (!this.excludedElements.contains(elmt.canoPath)) {
                if (elmt.isDir) {
                    config = this.directoriesConfig.get(elmt.canoPath);
                    this.recPath.setLength(0);
                    this.recPath.append(elmt.canoPath);
                    this.tmpPath = this.recPath.toString();
                    scanDirectory(elmt.file, config);
                } else {
                    long lastModified = elmt.file.lastModified();
                    if (lastModified != 0L) {
                        this.newScan.add(new ScanInfo(elmt.canoPath, lastModified, null));
                    }
                }
            }
        }
        Collections.sort(this.newScan);
    }

    /**
     * Report differences between the old and new scan.
     */
    private void reportChanges() {

        ScanInfo newElmt, oldElmt;
        String newPath, oldPath;

        int compare;
        int n = 0, newLength = this.newScan.size();
        int o = 0, oldLength = this.oldScan.size();

        while (n < newLength && o < oldLength) {

            newElmt = this.newScan.get(n);
            oldElmt = this.oldScan.get(o);
            newPath = newElmt.canoPath;
            oldPath = oldElmt.canoPath;
            compare = oldPath.compareTo(newPath);

            if (compare == 0) {
                if (newElmt.modified.compareTo(oldElmt.modified) != 0) {
                    this.inheritanceManager.callListeners(newElmt, CHANGED_ACTION);
                }
                n++; o++;
            } else if (compare < 0) {
                this.inheritanceManager.callListeners(oldElmt, DELETED_ACTION);
                o++;
            } else {
                this.inheritanceManager.callListeners(newElmt, ADDED_ACTION);
                n++;
            }
        }

        for (; o < oldLength; o++) {
            this.inheritanceManager.callListeners(this.oldScan.get(o), DELETED_ACTION);
        }
        for (; n < newLength; n++) {
            this.inheritanceManager.callListeners(this.newScan.get(n), ADDED_ACTION);
        }
    }

    /**
     * The task to repeat.
     */
    @Override
    protected synchronized void executeTask() {
        this.nbElmtsModified[ADDED_ACTION]   = 0;
        this.nbElmtsModified[CHANGED_ACTION] = 0;
        this.nbElmtsModified[DELETED_ACTION] = 0;
        this.lastScan = System.currentTimeMillis();
        
        scanElements();
        if (!this.firstScan || this.reportExistingFiles) {
            reportChanges();
        }

        this.oldScan.clear();
        this.oldScan = this.newScan;
        this.firstScan = false;
    }

    /**
     * List files of a directory.
     * @param dir The directory to scan
     * @param filter Filter applied on found files
     * @param list List of files accepted by the filter
     */
    private static void listFiles(final File dir, final FileFilter filter, final List<File> list) {
        File[] child = dir.listFiles();
        for (int i = 0; i < child.length; i++) {
            if (child[i].isDirectory()) {
                listFiles(child[i], filter, list);
            } else if (filter == null || filter.accept(child[i])) {
                list.add(child[i]);
            }
        }
    }

    /**
     * List files of a directory.
     * @param dirPath Path of the directory to scan
     * @param recursive If sub-directories are scanned
     * @param filter Filter applied on found files
     * @return The list of files accepted by the filter, or null
     *         if the file is not a directory
     */
    public static List<File> listFiles(final String dirPath, final boolean recursive, final FileFilter filter) {

        File dir = new File(dirPath);
        if (!dir.isDirectory()) {
            return null;
        }

        List<File> files = new ArrayList<File>();
        if (!recursive) {
            File[] child = dir.listFiles();
            for (int i = 0; i < child.length; i++) {
                if (filter == null || filter.accept(child[i])) {
                    files.add(child[i]);
                }
            }
        } else {
            listFiles(dir, filter, files);
        }
        return files;
    }

    /**
     * List directories of a directory.
     * @param dir The directory to scan
     * @param filter Filter applied on found files
     * @param list List of files accepted by the filter
     */
    private static void listDirectories(final File dir, final FileFilter filter, final List<File> list) {
        File[] child = dir.listFiles();
        for (int i = 0; i < child.length; i++) {
            if (child[i].isDirectory()) {
                if (filter == null || filter.accept(child[i])) {
                    list.add(child[i]);
                }
                listDirectories(child[i], filter, list);
            }
        }
    }

    /**
     * List directories of directory.
     * @param dirPath Path of the directory to scan
     * @param filter Filter applied on found files
     * @return The list of files accepted by the filter, or null
     *         if the file is not a directory
     */
    public static List<File> listDirectories(final String dirPath, final FileFilter filter) {

        File dir = new File(dirPath);
        if (!dir.isDirectory()) {
            return null;
        }
        List<File> files = new ArrayList<File>();
        files.add(dir);
        listDirectories(dir, filter, files);
        return files;
    }

    /**
     * List files and directories of a directory.
     * @param dir The directory to scan
     * @param filter Filter applied on found files
     * @param list List of files accepted by the filter
     */
    private static void listAll(final File dir, final FileFilter filter, final List<File> list) {
        File[] child = dir.listFiles();
        for (int i = 0; i < child.length; i++) {
            if (filter == null || filter.accept(child[i])) {
                list.add(child[i]);
            }
            if (child[i].isDirectory()) {
                listAll(child[i], filter, list);
            }
        }
    }

    /**
     * List files and directories of a directory.
     * @param dirPath Path of the directory to scan
     * @param recursive If sub-directories are scanned
     * @param filter Filter applied on found files
     * @return The list of files accepted by the filter
     */
    public static List<File> listAll(final String dirPath, final boolean recursive, final FileFilter filter) {

        File dir = new File(dirPath);
        if (!dir.isDirectory()) {
            return null;
        }

        List<File> files = new ArrayList<File>();
        files.add(dir);
        if (!recursive) {
            File[] child = dir.listFiles();
            for (int i = 0; i < child.length; i++) {
                if (filter == null || filter.accept(child[i])) {
                    files.add(child[i]);
                }
            }
        } else {
            listAll(dir, filter, files);
        }
        return files;
    }

    /**
     * Compare two scanners and call listeners of scanner
     * when differences are found.
     * @param baseScanner The base scanner
     * @param compScanner The scanner compared to the base scanner
     */
    public static void compareScanners(final Scanner baseScanner, final Scanner compScanner) {
        Scanner scanner = new Scanner();
        scanner.oldScan = baseScanner.newScan;
        scanner.newScan = compScanner.newScan;
        scanner.reportChanges();
    }

    /**
     * Compare content of two directories and call the listener
     * when differences are found.
     * @param baseDir The base directory
     * @param compDir The directory compared to the base directory
     * @param listener The listener called when differences are found
     */
    public static void compareDirectories(final String baseDir, final String compDir, final Listener listener) {
        compareDirectories(baseDir, compDir, listener, true, true, null);
    }

    /**
     * Compare content of two directories and call the listener
     * when differences are found between two directories.
     * @param baseDir The base directory
     * @param compDir The directory compared to the base directory
     * @param listener The listener called when differences are found
     * @param recursive Determine if the scan is recursive or not.
     * @param reportOnDirectory Determine if notifications are sended on directories
     * @param filter The filter which determine reported files
     */
    public static void compareDirectories(final String baseDir, final String compDir,
                                          final Listener listener, final boolean recursive,
                                          final boolean reportOnDirectory, final FileFilter filter) {
        Scanner scanner1 = new Scanner();
        Scanner scanner2 = new Scanner();
        scanner1.addScanDirectory(baseDir, listener, recursive, reportOnDirectory, filter);
        scanner2.addScanDirectory(compDir, listener, recursive, reportOnDirectory, filter);
        scanner1.scanElements();
        scanner2.scanElements();
        compareScanners(scanner1, scanner2);
    }
    
    /**
     * Get the canonical path of a file.
     * @param path Path of the file
     * @return The canonical path, or null if error.
     */
    private static String getCanonicalPath(final String path) {
        File file = new File(path);
        String canoPath = null;
        try {
            canoPath = file.getCanonicalPath();
        } catch (IOException ex) {
            Logger.getLogger(Scanner.class.getName()).log(Level.SEVERE, "Error while accessing file", ex);
        }
        return canoPath;
    }


    /****************************************************/
    /*                      Subclasses                  */
    /****************************************************/


    /**
     * Simple class representing scanned file.
     */
    static class ScannedElement {

        /**
         * The file of the scanned element.
         */
        private File file;

        /**
         * Canonical path of the scanned element.
         */
        private String canoPath;

        /**
         * Determine if the element is a file or a directory.
         */
        private boolean isDir;


        /**
         * Constructor.
         * @param canoPath Path of the file
         * @param isDir If element is a file or a directory
         */
        public ScannedElement(final String canoPath, final boolean isDir) {
            this.file = new File(canoPath);
            this.isDir = isDir;
            this.canoPath = canoPath;
        }

        /**
         * Get the scanned file.
         * @return The scanned file
         */
        public File getFile() {
            return this.file;
        }

        /**
         * Get the canonical path of a file.
         * @return Canonical path
         */
        public String getCanonicalPath() {
            return this.canoPath;
        }

        /**
         * Determine if element is a directory or not.
         * @return True if element is a directory, false otherwise
         */
        public boolean isDirectory() {
            return this.isDir;
        }

    }




    /**
     * Simple class representing information of a scanned element.
     */
    static class ScanInfo implements Comparable<ScanInfo> {

        /**
         * Canonical path of file.
         */
        private String canoPath;

        /**
         * The time the file was last modified.
         */
        private Long modified;

        /**
         * Configuration.
         */
        private DirConfig config;


        /**
         * Constructor.
         * @param canoPath Canonical path of file
         * @param modified The time the file was last modified
         * @param config Configuration of the element
         */
        public ScanInfo(final String canoPath, final long modified, final DirConfig config) {
            this.canoPath = canoPath;
            this.modified = new Long(modified);
            this.config = config;
        }

        /**
         * Comparable function.
         * @param se Other ScanElement
         * @return Integer
         */
        public int compareTo(final ScanInfo se) {
            return canoPath.compareTo(se.canoPath);
        }

        /**
         * Convert object to string.
         * @return String
         */
        @Override
        public String toString() {
            return "{" + canoPath + " :: " + modified + "}";
        }
    }


    /**
     * Configuration available on a directory.
     */
    static class DirConfig {

        /**
         * If the scan of directory is recursive.
         */
        private boolean recursive;

        /**
         * If events are reported on the directory.
         */
        private boolean reportOnDirectory;

        /**
         * File filter.
         */
        private FileFilter filter = null;

        /**
         * Link to listeners of the directory.
         */
        private List<Listener> listeners;

        /**
         * Configuration of the parent directory.
         * This field can change function of the global configuration.
         */
        private DirConfig parentConfig;

        /**
         * Canonical path of file.
         */
        private String canoPath;

        /**
         * Determine if configuration is initialised or not.
         */
        private boolean passive;


        /**
         * Constructor.
         * @param recursive Whether or not the scan is recursive
         * @param reportOnDirectory Whether or not events are reported on the directory
         * @param filter Filter applied on descendants files
         * @param listeners Listeners of the directory
         * @param canoPath The canonical path of element to configure
         */
        public DirConfig(final boolean recursive, final boolean reportOnDirectory, 
                         final FileFilter filter, final List<Listener> listeners,
                         final String canoPath) {
            this.recursive = recursive;
            this.reportOnDirectory = reportOnDirectory;
            this.filter = filter;
            this.listeners = listeners;
            this.parentConfig = null;
            this.canoPath = canoPath;
            this.passive = false;
        }

        /**
         * Constructor.
         * @param listeners Listeners of the directory
         * @param canoPath The canonical path of element to configure
         */
        public DirConfig(final List<Listener> listeners, final String canoPath) {
            this.listeners = listeners;
            this.parentConfig = null;
            this.canoPath = canoPath;
            this.passive = true;
        }

        /**
         * If the scan of directory is recursive.
         * @return Whether or not the scan is recursive
         */
        public boolean isRecursive() {
            return this.recursive;
        }

        /**
         * Set if the scan of directory is recursive.
         * @param recursive Whether or not the scan is recursive
         */
        public void setRecursive(final boolean recursive) {
            this.recursive = recursive;
        }

        /**
         * If events are reported on the directory.
         * @return Whether or not events are reported on the directory
         */
        public boolean getReportOnDirectory() {
            return this.reportOnDirectory;
        }

        /**
         * Set If events are reported on the directory.
         * @param reportOnDirectory Whether or not events are reported on the directory
         */
        public void setReportOnDirectory(final boolean reportOnDirectory) {
            this.reportOnDirectory = reportOnDirectory;
        }

        /**
         * Set a file filter.
         * @param filter Filter
         */
        public void setFilter(final FileFilter filter) {
            this.filter = filter;
        }

        /**
         * Get the file filter.
         * @return Filter
         */
        public FileFilter getFilter() {
            return this.filter;
        }

        /**
         * Set listeners of the directory.
         * @param listeners Listeners
         */
        public void setListeners(final List<Listener> listeners) {
            this.listeners = listeners;
        }

        /**
         * Get listeners of the directory.
         * @return Listeners
         */
        public List<Listener> getListeners() {
            return this.listeners;
        }

        /**
         * Set the parent directory configuration.
         * @param parentConfig The configuration
         */
        public void setParentConfig(final DirConfig parentConfig) {
            this.parentConfig = parentConfig;
        }

        /**
         * Get the parent directory configuration.
         * @return The configuration
         */
        public DirConfig getParentConfig() {
            return this.parentConfig;
        }

        /**
         * If the configuration is initialized or not.
         * @return True if the configuration is not initialized
         */
        public boolean isPassive() {
            return this.passive;
        }

        /**
         * Set if the configuration is initialized or not.
         * @param passive True if the configuration is not initialized
         */
        public void setPassive(final boolean passive) {
            this.passive = passive;
        }
    }
}
