/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: UnZipper.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.utils;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

/**
 * This class allow to unzip zip files.
 * @author Vincent Michaud
 */
public final class UnZipper {

    /**
     * The size of the buffer.
     */
    private static final int BUFFER_SIZE = 4096;

    /**
     * Constructor.
     */
    private UnZipper() {
    }

    /**
     * Unzip a zip file.
     * @param filePath The path of zip file
     * @throws ZipException If the file can not be unzipped
     * @throws IOException If the file can not be unzipped
     */
    public static void unzip(final String filePath) throws ZipException, IOException {
        unzip(new File(filePath), new File("."), true, false);
    }

    /**
     * Unzip a zip file.
     * @param file The zip file
     * @throws ZipException If the file can not be unzipped
     * @throws IOException If the file can not be unzipped
     */
    public static void unzip(final File file) throws ZipException, IOException {
        unzip(file, new File("."), true, false);
    }

    /**
     * Unzip a zip file.
     * @param filePath The path of zip file
     * @param dirPath The path of directory to unzip the file
     * @param erase If files are erased or not
     * @param removeRootFolder Determine, if there is a root folder, if it is removed
     * @throws ZipException If the file can not be unzipped
     * @throws IOException If the file can not be unzipped
     */
    public static void unzip(final String filePath, final String dirPath, 
                             final boolean erase, final boolean removeRootFolder)
            throws ZipException, IOException {
        unzip(new ZipFile(filePath), new File(dirPath), erase, removeRootFolder);
    }

    /**
     * Unzip a zip file.
     * @param file The zip file
     * @param dirFile The directory to unzip the file
     * @param erase If files are erased or not
     * @param removeRootFolder Determine, if there is a root folder, if it is removed
     * @throws ZipException If the file can not be unzipped
     * @throws IOException If the file can not be unzipped
     */
    public static void unzip(final File file, final File dirFile, final boolean erase,
                             final boolean removeRootFolder) throws ZipException, IOException {
        unzip(new ZipFile(file, ZipFile.OPEN_READ), dirFile, erase, removeRootFolder);
    }

    /**
     * Unzip a zip file.
     * @param zipFile The zip file
     * @param dirFile The directory to unzip the file
     * @param erase If files are erased or not
     * @param removeRootFolder Determine, if there is a root folder, if it is removed
     * @throws ZipException If the file can not be unzipped
     * @throws IOException If the file can not be unzipped
     */
    public static void unzip(final ZipFile zipFile, final File dirFile, final boolean erase,
                             final boolean removeRootFolder) throws ZipException, IOException {
        if (!dirFile.exists()) {
            dirFile.mkdirs();
        }
        if (dirFile.isDirectory()) {
            Enumeration<ZipEntry> entries = (Enumeration<ZipEntry>) zipFile.entries();
            List<ZipEntry> fileEntries = new LinkedList();
            String dirPath = dirFile.getAbsolutePath() + File.separator;
            InputStream is;
            OutputStream os;
            ZipEntry entry;
            File entryFile;

            int index = 0;
            if (removeRootFolder) {
                index = hasRootFolder(zipFile);
            }

            while (entries.hasMoreElements()) {
                entry = entries.nextElement();

                if (entry.isDirectory()) {
                    entryFile = new File(dirPath + entry.getName().substring(index));
                    if (!entryFile.exists()) {
                        entryFile.mkdirs();
                    }
                } else {
                    fileEntries.add(entry);
                }
            }

            if (erase) {
                for (ZipEntry zipEntry : fileEntries) {
                    entryFile = new File(dirPath + zipEntry.getName().substring(index));
                    is = zipFile.getInputStream(zipEntry);
                    os = new BufferedOutputStream(new FileOutputStream(entryFile));
                    copyStream(is, os);
                }
            } else {
                for (ZipEntry zipEntry : fileEntries) {
                    entryFile = new File(dirPath + zipEntry.getName().substring(index));
                    if (!entryFile.exists()) {
                        is = zipFile.getInputStream(zipEntry);
                        os = new BufferedOutputStream(new FileOutputStream(entryFile));
                        copyStream(is, os);
                    }
                }
            }
        }
    }

    /**
     * Determine if a ZIP file has a root folder.
     * @param zipFile The ZIP file
     * @return 0 if the file has not root folder,
     *         the length of the root folder otherwise
     */
    private static int hasRootFolder(final ZipFile zipFile) {
        Enumeration<ZipEntry> entries = (Enumeration<ZipEntry>) zipFile.entries();
        int indexRootPath;
        String rootPath;
        ZipEntry entry;

        if (entries.hasMoreElements()) {
            entry = entries.nextElement();
            String name = entry.getName();
            indexRootPath = name.indexOf("/", 1);

            if (indexRootPath != -1) {
                indexRootPath++;
                rootPath = name.substring(0, indexRootPath);
                while (entries.hasMoreElements()) {
                    entry = entries.nextElement();
                    if (!entry.getName().startsWith(rootPath)) {
                        return 0;
                    }
                }
                return indexRootPath;
            }
        }
        return 0;
    }

    /**
     * Copy an input stream into an output stream.
     * @param is The input stream
     * @param os The output stream
     * @throws IOException If an error occurs
     */
    public static void copyStream(final InputStream is, final OutputStream os) throws IOException {
        byte[] buffer = new byte[BUFFER_SIZE];
        try {
            int length = is.read(buffer);
            while (length > 0) {
                os.write(buffer, 0, length);
                length = is.read(buffer);
            }
        } finally {
            is.close();
            os.close();
        }
    }
}
