/**
 * EasyBeans
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: UrlUtils.java 5360 2010-02-24 14:29:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.maven.plugin.deployment.utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;


/**
 * Class for download files with an URL.
 * @author Sebastien Desbouchages
 */
public final class UrlUtils {

    /**
     * Utility class : Private constructor.
     */
    private UrlUtils() {
    }

    /**
     * The length of an array of bytes.
     */
    private static final int ARRAY_LENGTH = 1024;


    /**
     * Download a file from an URL.
     * @param spec The string to parse as a URL
     * @return Downloaded file
     * @throws IOException Error while downloading
     */
    public static File downloadFile(final String spec) throws IOException  {
        URL url = new URL(spec);
        return downloadFile(url);
    }


    /**
     * Download a file from an URL.
     * @param url The URL
     * @return Downloaded file
     * @throws IOException Error while downloading
     */
    public static File downloadFile(final URL url) throws IOException  {
        File file = getFile(url);
        downloadFile(url, file);
        return file;
    }



    /**
     * Download a file in a File from an URL.
     * @param spec The string to parse as a URL
     * @param dest The File
     * @throws IOException Error while downloading
     */
    public static void downloadFile(final String spec, final File dest) throws IOException  {
        URL url = new URL(spec);
        downloadFile(url, dest);
    }



    /**
     * Download a file in a File from an URL.
     * @param url The URL
     * @param dest The File
     * @throws IOException Error while downloading
     */
    public static void downloadFile(final URL url, final File dest) throws IOException {
        FileOutputStream fos = null;
        InputStream in = null;
        URLConnection connection = url.openConnection();

        if (connection.getContentLength() == -1) {
            throw new IOException("Connection error.");
        }

        in = connection.getInputStream();

        try {
            fos = new FileOutputStream(dest);
            byte[] buff = new byte[ARRAY_LENGTH];
            int l = in.read(buff);
            while (l > 0) {
                fos.write(buff, 0, l);
                l = in.read(buff);
            }
        } finally {
            in.close();
            fos.flush();
            fos.close();
        }
    }


    /**
     * Create a file from an URL.
     * @param url The URL
     * @return New file
     */
    private static File getFile(final URL url) {
        String fileName = url.getFile();
        fileName = fileName.substring(fileName.lastIndexOf('/') + 1);
        return new File(fileName);
    }
}
